## Generated by h2py from sql.h
#ODBCVER = 0x0351
#SQL_NULL_DATA = (-1)
#SQL_DATA_AT_EXEC = (-2)
#SQL_SUCCESS = 0
#SQL_SUCCESS_WITH_INFO = 1
#SQL_NO_DATA = 100
#SQL_ERROR = (-1)
#SQL_INVALID_HANDLE = (-2)
#SQL_STILL_EXECUTING = 2
#SQL_NEED_DATA = 99
#def SQL_SUCCEEDED(rc): return (((rc)&(~1))==0)
#
#SQL_NTS = (-3)
#SQL_NTSL = (-3L)
#SQL_MAX_MESSAGE_LENGTH = 512
#SQL_DATE_LEN = 10
#SQL_TIME_LEN = 8
#SQL_TIMESTAMP_LEN = 19
#SQL_HANDLE_ENV = 1
#SQL_HANDLE_DBC = 2
#SQL_HANDLE_STMT = 3
#SQL_HANDLE_DESC = 4
#SQL_ATTR_OUTPUT_NTS = 10001
#SQL_ATTR_AUTO_IPD = 10001
#SQL_ATTR_METADATA_ID = 10014
#SQL_ATTR_APP_ROW_DESC = 10010
#SQL_ATTR_APP_PARAM_DESC = 10011
#SQL_ATTR_IMP_ROW_DESC = 10012
#SQL_ATTR_IMP_PARAM_DESC = 10013
#SQL_ATTR_CURSOR_SCROLLABLE = (-1)
#SQL_ATTR_CURSOR_SENSITIVITY = (-2)
#SQL_NONSCROLLABLE = 0
#SQL_SCROLLABLE = 1
#SQL_DESC_COUNT = 1001
#SQL_DESC_TYPE = 1002
#SQL_DESC_LENGTH = 1003
#SQL_DESC_OCTET_LENGTH_PTR = 1004
#SQL_DESC_PRECISION = 1005
#SQL_DESC_SCALE = 1006
#SQL_DESC_DATETIME_INTERVAL_CODE = 1007
#SQL_DESC_NULLABLE = 1008
#SQL_DESC_INDICATOR_PTR = 1009
#SQL_DESC_DATA_PTR = 1010
#SQL_DESC_NAME = 1011
#SQL_DESC_UNNAMED = 1012
#SQL_DESC_OCTET_LENGTH = 1013
#SQL_DESC_ALLOC_TYPE = 1099
#SQL_DIAG_RETURNCODE = 1
#SQL_DIAG_NUMBER = 2
#SQL_DIAG_ROW_COUNT = 3
#SQL_DIAG_SQLSTATE = 4
#SQL_DIAG_NATIVE = 5
#SQL_DIAG_MESSAGE_TEXT = 6
#SQL_DIAG_DYNAMIC_FUNCTION = 7
#SQL_DIAG_CLASS_ORIGIN = 8
#SQL_DIAG_SUBCLASS_ORIGIN = 9
#SQL_DIAG_CONNECTION_NAME = 10
#SQL_DIAG_SERVER_NAME = 11
#SQL_DIAG_DYNAMIC_FUNCTION_CODE = 12
#SQL_DIAG_ALTER_DOMAIN = 3
#SQL_DIAG_ALTER_TABLE = 4
#SQL_DIAG_CALL = 7
#SQL_DIAG_CREATE_ASSERTION = 6
#SQL_DIAG_CREATE_CHARACTER_SET = 8
#SQL_DIAG_CREATE_COLLATION = 10
#SQL_DIAG_CREATE_DOMAIN = 23
#SQL_DIAG_CREATE_INDEX = (-1)
#SQL_DIAG_CREATE_SCHEMA = 64
#SQL_DIAG_CREATE_TABLE = 77
#SQL_DIAG_CREATE_TRANSLATION = 79
#SQL_DIAG_CREATE_VIEW = 84
#SQL_DIAG_DELETE_WHERE = 19
#SQL_DIAG_DROP_ASSERTION = 24
#SQL_DIAG_DROP_CHARACTER_SET = 25
#SQL_DIAG_DROP_COLLATION = 26
#SQL_DIAG_DROP_DOMAIN = 27
#SQL_DIAG_DROP_INDEX = (-2)
#SQL_DIAG_DROP_SCHEMA = 31
#SQL_DIAG_DROP_TABLE = 32
#SQL_DIAG_DROP_TRANSLATION = 33
#SQL_DIAG_DROP_VIEW = 36
#SQL_DIAG_DYNAMIC_DELETE_CURSOR = 38
#SQL_DIAG_DYNAMIC_UPDATE_CURSOR = 81
#SQL_DIAG_GRANT = 48
#SQL_DIAG_INSERT = 50
#SQL_DIAG_REVOKE = 59
#SQL_DIAG_SELECT_CURSOR = 85
#SQL_DIAG_UNKNOWN_STATEMENT = 0
#SQL_DIAG_UPDATE_WHERE = 82
#SQL_UNKNOWN_TYPE = 0
#SQL_CHAR = 1
#SQL_NUMERIC = 2
#SQL_DECIMAL = 3
#SQL_INTEGER = 4
#SQL_SMALLINT = 5
#SQL_FLOAT = 6
#SQL_REAL = 7
#SQL_DOUBLE = 8
#SQL_DATETIME = 9
#SQL_VARCHAR = 12
#SQL_TYPE_DATE = 91
#SQL_TYPE_TIME = 92
#SQL_TYPE_TIMESTAMP = 93
#SQL_UNSPECIFIED = 0
#SQL_INSENSITIVE = 1
#SQL_SENSITIVE = 2
#SQL_ALL_TYPES = 0
#SQL_DEFAULT = 99
#SQL_ARD_TYPE = (-99)
#SQL_CODE_DATE = 1
#SQL_CODE_TIME = 2
#SQL_CODE_TIMESTAMP = 3
#SQL_FALSE = 0
#SQL_TRUE = 1
#SQL_NO_NULLS = 0
#SQL_NULLABLE = 1
#SQL_NULLABLE_UNKNOWN = 2
#SQL_PRED_NONE = 0
#SQL_PRED_CHAR = 1
#SQL_PRED_BASIC = 2
#SQL_NAMED = 0
#SQL_UNNAMED = 1
#SQL_DESC_ALLOC_AUTO = 1
#SQL_DESC_ALLOC_USER = 2
#SQL_CLOSE = 0
#SQL_DROP = 1
#SQL_UNBIND = 2
#SQL_RESET_PARAMS = 3
#SQL_FETCH_NEXT = 1
#SQL_FETCH_FIRST = 2
#SQL_FETCH_LAST = 3
#SQL_FETCH_PRIOR = 4
#SQL_FETCH_ABSOLUTE = 5
#SQL_FETCH_RELATIVE = 6
#SQL_COMMIT = 0
#SQL_ROLLBACK = 1
#SQL_NULL_HENV = 0
#SQL_NULL_HDBC = 0
#SQL_NULL_HSTMT = 0
#SQL_NULL_HDESC = 0
#SQL_NULL_DESC = 0
#SQL_NULL_HANDLE = 0L
#SQL_SCOPE_CURROW = 0
#SQL_SCOPE_TRANSACTION = 1
#SQL_SCOPE_SESSION = 2
#SQL_PC_UNKNOWN = 0
#SQL_PC_NON_PSEUDO = 1
#SQL_PC_PSEUDO = 2
#SQL_ROW_IDENTIFIER = 1
#SQL_INDEX_UNIQUE = 0
#SQL_INDEX_ALL = 1
#SQL_INDEX_CLUSTERED = 1
#SQL_INDEX_HASHED = 2
#SQL_INDEX_OTHER = 3
#SQL_API_SQLALLOCCONNECT = 1
#SQL_API_SQLALLOCENV = 2
#SQL_API_SQLALLOCHANDLE = 1001
#SQL_API_SQLALLOCSTMT = 3
#SQL_API_SQLBINDCOL = 4
#SQL_API_SQLBINDPARAM = 1002
#SQL_API_SQLCANCEL = 5
#SQL_API_SQLCLOSECURSOR = 1003
#SQL_API_SQLCOLATTRIBUTE = 6
#SQL_API_SQLCOLUMNS = 40
#SQL_API_SQLCONNECT = 7
#SQL_API_SQLCOPYDESC = 1004
#SQL_API_SQLDATASOURCES = 57
#SQL_API_SQLDESCRIBECOL = 8
#SQL_API_SQLDISCONNECT = 9
#SQL_API_SQLENDTRAN = 1005
#SQL_API_SQLERROR = 10
#SQL_API_SQLEXECDIRECT = 11
#SQL_API_SQLEXECUTE = 12
#SQL_API_SQLFETCH = 13
#SQL_API_SQLFETCHSCROLL = 1021
#SQL_API_SQLFREECONNECT = 14
#SQL_API_SQLFREEENV = 15
#SQL_API_SQLFREEHANDLE = 1006
#SQL_API_SQLFREESTMT = 16
#SQL_API_SQLGETCONNECTATTR = 1007
#SQL_API_SQLGETCONNECTOPTION = 42
#SQL_API_SQLGETCURSORNAME = 17
#SQL_API_SQLGETDATA = 43
#SQL_API_SQLGETDESCFIELD = 1008
#SQL_API_SQLGETDESCREC = 1009
#SQL_API_SQLGETDIAGFIELD = 1010
#SQL_API_SQLGETDIAGREC = 1011
#SQL_API_SQLGETENVATTR = 1012
#SQL_API_SQLGETFUNCTIONS = 44
#SQL_API_SQLGETINFO = 45
#SQL_API_SQLGETSTMTATTR = 1014
#SQL_API_SQLGETSTMTOPTION = 46
#SQL_API_SQLGETTYPEINFO = 47
#SQL_API_SQLNUMRESULTCOLS = 18
#SQL_API_SQLPARAMDATA = 48
#SQL_API_SQLPREPARE = 19
#SQL_API_SQLPUTDATA = 49
#SQL_API_SQLROWCOUNT = 20
#SQL_API_SQLSETCONNECTATTR = 1016
#SQL_API_SQLSETCONNECTOPTION = 50
#SQL_API_SQLSETCURSORNAME = 21
#SQL_API_SQLSETDESCFIELD = 1017
#SQL_API_SQLSETDESCREC = 1018
#SQL_API_SQLSETENVATTR = 1019
#SQL_API_SQLSETPARAM = 22
#SQL_API_SQLSETSTMTATTR = 1020
#SQL_API_SQLSETSTMTOPTION = 51
#SQL_API_SQLSPECIALCOLUMNS = 52
#SQL_API_SQLSTATISTICS = 53
#SQL_API_SQLTABLES = 54
#SQL_API_SQLTRANSACT = 23
#SQL_MAX_DRIVER_CONNECTIONS = 0
#SQL_MAXIMUM_DRIVER_CONNECTIONS = SQL_MAX_DRIVER_CONNECTIONS
#SQL_MAX_CONCURRENT_ACTIVITIES = 1
#SQL_MAXIMUM_CONCURRENT_ACTIVITIES = SQL_MAX_CONCURRENT_ACTIVITIES
#SQL_DATA_SOURCE_NAME = 2
#SQL_FETCH_DIRECTION = 8
#SQL_SERVER_NAME = 13
#SQL_SEARCH_PATTERN_ESCAPE = 14
#SQL_DBMS_NAME = 17
#SQL_DBMS_VER = 18
#SQL_ACCESSIBLE_TABLES = 19
#SQL_ACCESSIBLE_PROCEDURES = 20
#SQL_CURSOR_COMMIT_BEHAVIOR = 23
#SQL_DATA_SOURCE_READ_ONLY = 25
#SQL_DEFAULT_TXN_ISOLATION = 26
#SQL_IDENTIFIER_CASE = 28
#SQL_IDENTIFIER_QUOTE_CHAR = 29
#SQL_MAX_COLUMN_NAME_LEN = 30
#SQL_MAXIMUM_COLUMN_NAME_LENGTH = SQL_MAX_COLUMN_NAME_LEN
#SQL_MAX_CURSOR_NAME_LEN = 31
#SQL_MAXIMUM_CURSOR_NAME_LENGTH = SQL_MAX_CURSOR_NAME_LEN
#SQL_MAX_SCHEMA_NAME_LEN = 32
#SQL_MAXIMUM_SCHEMA_NAME_LENGTH = SQL_MAX_SCHEMA_NAME_LEN
#SQL_MAX_CATALOG_NAME_LEN = 34
#SQL_MAXIMUM_CATALOG_NAME_LENGTH = SQL_MAX_CATALOG_NAME_LEN
#SQL_MAX_TABLE_NAME_LEN = 35
#SQL_SCROLL_CONCURRENCY = 43
#SQL_TXN_CAPABLE = 46
#SQL_TRANSACTION_CAPABLE = SQL_TXN_CAPABLE
#SQL_USER_NAME = 47
#SQL_TXN_ISOLATION_OPTION = 72
#SQL_TRANSACTION_ISOLATION_OPTION = SQL_TXN_ISOLATION_OPTION
#SQL_INTEGRITY = 73
#SQL_GETDATA_EXTENSIONS = 81
#SQL_NULL_COLLATION = 85
#SQL_ALTER_TABLE = 86
#SQL_ORDER_BY_COLUMNS_IN_SELECT = 90
#SQL_SPECIAL_CHARACTERS = 94
#SQL_MAX_COLUMNS_IN_GROUP_BY = 97
#SQL_MAXIMUM_COLUMNS_IN_GROUP_BY = SQL_MAX_COLUMNS_IN_GROUP_BY
#SQL_MAX_COLUMNS_IN_INDEX = 98
#SQL_MAXIMUM_COLUMNS_IN_INDEX = SQL_MAX_COLUMNS_IN_INDEX
#SQL_MAX_COLUMNS_IN_ORDER_BY = 99
#SQL_MAXIMUM_COLUMNS_IN_ORDER_BY = SQL_MAX_COLUMNS_IN_ORDER_BY
#SQL_MAX_COLUMNS_IN_SELECT = 100
#SQL_MAXIMUM_COLUMNS_IN_SELECT = SQL_MAX_COLUMNS_IN_SELECT
#SQL_MAX_COLUMNS_IN_TABLE = 101
#SQL_MAX_INDEX_SIZE = 102
#SQL_MAXIMUM_INDEX_SIZE = SQL_MAX_INDEX_SIZE
#SQL_MAX_ROW_SIZE = 104
#SQL_MAXIMUM_ROW_SIZE = SQL_MAX_ROW_SIZE
#SQL_MAX_STATEMENT_LEN = 105
#SQL_MAXIMUM_STATEMENT_LENGTH = SQL_MAX_STATEMENT_LEN
#SQL_MAX_TABLES_IN_SELECT = 106
#SQL_MAXIMUM_TABLES_IN_SELECT = SQL_MAX_TABLES_IN_SELECT
#SQL_MAX_USER_NAME_LEN = 107
#SQL_MAXIMUM_USER_NAME_LENGTH = SQL_MAX_USER_NAME_LEN
#SQL_OJ_CAPABILITIES = 115
#SQL_OUTER_JOIN_CAPABILITIES = SQL_OJ_CAPABILITIES
#SQL_XOPEN_CLI_YEAR = 10000
#SQL_CURSOR_SENSITIVITY = 10001
#SQL_DESCRIBE_PARAMETER = 10002
#SQL_CATALOG_NAME = 10003
#SQL_COLLATION_SEQ = 10004
#SQL_MAX_IDENTIFIER_LEN = 10005
#SQL_MAXIMUM_IDENTIFIER_LENGTH = SQL_MAX_IDENTIFIER_LEN
#SQL_AT_ADD_COLUMN = 0x00000001L
#SQL_AT_DROP_COLUMN = 0x00000002L
#SQL_AT_ADD_CONSTRAINT = 0x00000008L
#SQL_AM_NONE = 0
#SQL_AM_CONNECTION = 1
#SQL_AM_STATEMENT = 2
#SQL_CB_DELETE = 0
#SQL_CB_CLOSE = 1
#SQL_CB_PRESERVE = 2
#SQL_FD_FETCH_NEXT = 0x00000001L
#SQL_FD_FETCH_FIRST = 0x00000002L
#SQL_FD_FETCH_LAST = 0x00000004L
#SQL_FD_FETCH_PRIOR = 0x00000008L
#SQL_FD_FETCH_ABSOLUTE = 0x00000010L
#SQL_FD_FETCH_RELATIVE = 0x00000020L
#SQL_GD_ANY_COLUMN = 0x00000001L
#SQL_GD_ANY_ORDER = 0x00000002L
#SQL_IC_UPPER = 1
#SQL_IC_LOWER = 2
#SQL_IC_SENSITIVE = 3
#SQL_IC_MIXED = 4
#SQL_OJ_LEFT = 0x00000001L
#SQL_OJ_RIGHT = 0x00000002L
#SQL_OJ_FULL = 0x00000004L
#SQL_OJ_NESTED = 0x00000008L
#SQL_OJ_NOT_ORDERED = 0x00000010L
#SQL_OJ_INNER = 0x00000020L
#SQL_OJ_ALL_COMPARISON_OPS = 0x00000040L
#SQL_SCCO_READ_ONLY = 0x00000001L
#SQL_SCCO_LOCK = 0x00000002L
#SQL_SCCO_OPT_ROWVER = 0x00000004L
#SQL_SCCO_OPT_VALUES = 0x00000008L
#SQL_TC_NONE = 0
#SQL_TC_DML = 1
#SQL_TC_ALL = 2
#SQL_TC_DDL_COMMIT = 3
#SQL_TC_DDL_IGNORE = 4
#SQL_TXN_READ_UNCOMMITTED = 0x00000001L
#SQL_TRANSACTION_READ_UNCOMMITTED = SQL_TXN_READ_UNCOMMITTED
#SQL_TXN_READ_COMMITTED = 0x00000002L
#SQL_TRANSACTION_READ_COMMITTED = SQL_TXN_READ_COMMITTED
#SQL_TXN_REPEATABLE_READ = 0x00000004L
#SQL_TRANSACTION_REPEATABLE_READ = SQL_TXN_REPEATABLE_READ
#SQL_TXN_SERIALIZABLE = 0x00000008L
#SQL_TRANSACTION_SERIALIZABLE = SQL_TXN_SERIALIZABLE
#SQL_NC_HIGH = 0
#SQL_NC_LOW = 1
## Generated by h2py from sqlext.h
#SQLTables_TABLE_CATALOG = 1
#SQLTables_TABLE_SCHEM = 2
#SQLTables_TABLE_NAME = 3
#SQLTables_TABLE_TYPE = 4
#SQLTables_REMARKS = 5
#SQLColumns_TABLE_CAT = 1
#SQLColumns_TABLE_SCHEM = 2
#SQLColumns_TABLE_NAME = 3
#SQLColumns_COLUMN_NAME = 4
#SQLColumns_DATA_TYPE = 5
#SQLColumns_TYPE_NAME = 6
#SQLColumns_COLUMN_SIZE = 7
#SQLColumns_BUFFER_LENGTH = 8
#SQLColumns_DECIMAL_DIGITS = 9
#SQLColumns_NUM_PREC_RADIX = 10
#SQLColumns_NULLABLE = 11
#SQLColumns_REMARKS = 12
#SQLColumns_COLUMN_DEF = 13
#SQLColumns_SQL_DATA_TYPE = 14
#SQLColumns_SQL_DATETIME_SUB = 15
#SQLColumns_CHAR_OCTET_LENGTH = 16
#SQLColumns_ORDINAL_POSITION = 17
#SQLColumns_IS_NULLABLE = 18
#SQL_SPEC_MAJOR = 3
#SQL_SPEC_MINOR = 52
#SQL_SPEC_STRING = "03.52"
#SQL_SQLSTATE_SIZE = 5
#SQL_MAX_DSN_LENGTH = 32
#SQL_MAX_OPTION_STRING_LENGTH = 256
#SQL_NO_DATA_FOUND = 100
#SQL_HANDLE_SENV = 5
#SQL_ATTR_ODBC_VERSION = 200
#SQL_ATTR_CONNECTION_POOLING = 201
#SQL_ATTR_CP_MATCH = 202
#SQL_ATTR_UNIXODBC_SYSPATH = 65001
#SQL_ATTR_UNIXODBC_VERSION = 65002
#SQL_ATTR_UNIXODBC_ENVATTR = 65003
#SQL_ACCESS_MODE = 101
#SQL_AUTOCOMMIT = 102
#SQL_LOGIN_TIMEOUT = 103
#SQL_OPT_TRACE = 104
#SQL_OPT_TRACEFILE = 105
#SQL_TRANSLATE_DLL = 106
#SQL_TRANSLATE_OPTION = 107
#SQL_TXN_ISOLATION = 108
#SQL_CURRENT_QUALIFIER = 109
#SQL_ODBC_CURSORS = 110
#SQL_QUIET_MODE = 111
#SQL_PACKET_SIZE = 112
#SQL_ATTR_ACCESS_MODE = SQL_ACCESS_MODE
#SQL_ATTR_AUTOCOMMIT = SQL_AUTOCOMMIT
#SQL_ATTR_CONNECTION_TIMEOUT = 113
#SQL_ATTR_CURRENT_CATALOG = SQL_CURRENT_QUALIFIER
#SQL_ATTR_DISCONNECT_BEHAVIOR = 114
#SQL_ATTR_ENLIST_IN_DTC = 1207
#SQL_ATTR_ENLIST_IN_XA = 1208
#SQL_ATTR_LOGIN_TIMEOUT = SQL_LOGIN_TIMEOUT
#SQL_ATTR_ODBC_CURSORS = SQL_ODBC_CURSORS
#SQL_ATTR_PACKET_SIZE = SQL_PACKET_SIZE
#SQL_ATTR_QUIET_MODE = SQL_QUIET_MODE
#SQL_ATTR_TRACE = SQL_OPT_TRACE
#SQL_ATTR_TRACEFILE = SQL_OPT_TRACEFILE
#SQL_ATTR_TRANSLATE_LIB = SQL_TRANSLATE_DLL
#SQL_ATTR_TRANSLATE_OPTION = SQL_TRANSLATE_OPTION
#SQL_ATTR_TXN_ISOLATION = SQL_TXN_ISOLATION
#SQL_ATTR_CONNECTION_DEAD = 1209
#SQL_ATTR_DRIVER_THREADING = 1028
#SQL_ATTR_ANSI_APP = 115
#SQL_CONNECT_OPT_DRVR_START = 1000
#SQL_CONN_OPT_MAX = SQL_PACKET_SIZE
#SQL_CONN_OPT_MIN = SQL_ACCESS_MODE
#SQL_OPT_TRACE_FILE_DEFAULT = "/tmp/SQL.LOG"
#SQL_DTC_DONE = 0L
#SQL_CD_TRUE = 1L
#SQL_CD_FALSE = 0L
#SQL_AA_TRUE = 1L
#SQL_AA_FALSE = 0L
#SQL_QUERY_TIMEOUT = 0
#SQL_MAX_ROWS = 1
#SQL_NOSCAN = 2
#SQL_MAX_LENGTH = 3
#SQL_ASYNC_ENABLE = 4
#SQL_BIND_TYPE = 5
#SQL_CURSOR_TYPE = 6
#SQL_CONCURRENCY = 7
#SQL_KEYSET_SIZE = 8
#SQL_ROWSET_SIZE = 9
#SQL_SIMULATE_CURSOR = 10
#SQL_RETRIEVE_DATA = 11
#SQL_USE_BOOKMARKS = 12
#SQL_GET_BOOKMARK = 13
#SQL_ROW_NUMBER = 14
#SQL_ATTR_ASYNC_ENABLE = 4
#SQL_ATTR_CONCURRENCY = SQL_CONCURRENCY
#SQL_ATTR_CURSOR_TYPE = SQL_CURSOR_TYPE
#SQL_ATTR_ENABLE_AUTO_IPD = 15
#SQL_ATTR_FETCH_BOOKMARK_PTR = 16
#SQL_ATTR_KEYSET_SIZE = SQL_KEYSET_SIZE
#SQL_ATTR_MAX_LENGTH = SQL_MAX_LENGTH
#SQL_ATTR_MAX_ROWS = SQL_MAX_ROWS
#SQL_ATTR_NOSCAN = SQL_NOSCAN
#SQL_ATTR_PARAM_BIND_OFFSET_PTR = 17
#SQL_ATTR_PARAM_BIND_TYPE = 18
#SQL_ATTR_PARAM_OPERATION_PTR = 19
#SQL_ATTR_PARAM_STATUS_PTR = 20
#SQL_ATTR_PARAMS_PROCESSED_PTR = 21
#SQL_ATTR_PARAMSET_SIZE = 22
#SQL_ATTR_QUERY_TIMEOUT = SQL_QUERY_TIMEOUT
#SQL_ATTR_RETRIEVE_DATA = SQL_RETRIEVE_DATA
#SQL_ATTR_ROW_BIND_OFFSET_PTR = 23
#SQL_ATTR_ROW_BIND_TYPE = SQL_BIND_TYPE
#SQL_ATTR_ROW_NUMBER = SQL_ROW_NUMBER
#SQL_ATTR_ROW_OPERATION_PTR = 24
#SQL_ATTR_ROW_STATUS_PTR = 25
#SQL_ATTR_ROWS_FETCHED_PTR = 26
#SQL_ATTR_ROW_ARRAY_SIZE = 27
#SQL_ATTR_SIMULATE_CURSOR = SQL_SIMULATE_CURSOR
#SQL_ATTR_USE_BOOKMARKS = SQL_USE_BOOKMARKS
#SQL_STMT_OPT_MAX = SQL_ROW_NUMBER
#SQL_STMT_OPT_MIN = SQL_QUERY_TIMEOUT
#SQL_IS_POINTER = (-4)
#SQL_IS_UINTEGER = (-5)
#SQL_IS_INTEGER = (-6)
#SQL_IS_USMALLINT = (-7)
#SQL_IS_SMALLINT = (-8)
#SQL_CONCUR_READ_ONLY = 1
#SQL_CONCUR_LOCK = 2
#SQL_CONCUR_ROWVER = 3
#SQL_CONCUR_VALUES = 4
#SQL_CONCUR_DEFAULT = SQL_CONCUR_READ_ONLY
#SQL_DESC_ARRAY_SIZE = 20
#SQL_DESC_ARRAY_STATUS_PTR = 21
#SQL_DESC_BASE_COLUMN_NAME = 22
#SQL_DESC_BASE_TABLE_NAME = 23
#SQL_DESC_BIND_OFFSET_PTR = 24
#SQL_DESC_BIND_TYPE = 25
#SQL_DESC_DATETIME_INTERVAL_PRECISION = 26
#SQL_DESC_LITERAL_PREFIX = 27
#SQL_DESC_LITERAL_SUFFIX = 28
#SQL_DESC_LOCAL_TYPE_NAME = 29
#SQL_DESC_MAXIMUM_SCALE = 30
#SQL_DESC_MINIMUM_SCALE = 31
#SQL_DESC_NUM_PREC_RADIX = 32
#SQL_DESC_PARAMETER_TYPE = 33
#SQL_DESC_ROWS_PROCESSED_PTR = 34
#SQL_DESC_ROWVER = 35
#SQL_DIAG_CURSOR_ROW_COUNT = (-1249)
#SQL_DIAG_ROW_NUMBER = (-1248)
#SQL_DIAG_COLUMN_NUMBER = (-1247)
#SQL_DATE = 9
#SQL_INTERVAL = 10
#SQL_TIME = 10
#SQL_TIMESTAMP = 11
#SQL_LONGVARCHAR = (-1)
#SQL_BINARY = (-2)
#SQL_VARBINARY = (-3)
#SQL_LONGVARBINARY = (-4)
#SQL_BIGINT = (-5)
#SQL_TINYINT = (-6)
#SQL_BIT = (-7)
#SQL_GUID = (-11)
#SQL_CODE_YEAR = 1
#SQL_CODE_MONTH = 2
#SQL_CODE_DAY = 3
#SQL_CODE_HOUR = 4
#SQL_CODE_MINUTE = 5
#SQL_CODE_SECOND = 6
#SQL_CODE_YEAR_TO_MONTH = 7
#SQL_CODE_DAY_TO_HOUR = 8
#SQL_CODE_DAY_TO_MINUTE = 9
#SQL_CODE_DAY_TO_SECOND = 10
#SQL_CODE_HOUR_TO_MINUTE = 11
#SQL_CODE_HOUR_TO_SECOND = 12
#SQL_CODE_MINUTE_TO_SECOND = 13
#SQL_INTERVAL_YEAR = (100 + SQL_CODE_YEAR)
#SQL_INTERVAL_MONTH = (100 + SQL_CODE_MONTH)
#SQL_INTERVAL_DAY = (100 + SQL_CODE_DAY)
#SQL_INTERVAL_HOUR = (100 + SQL_CODE_HOUR)
#SQL_INTERVAL_MINUTE = (100 + SQL_CODE_MINUTE)
#SQL_INTERVAL_SECOND = (100 + SQL_CODE_SECOND)
#SQL_INTERVAL_YEAR_TO_MONTH = (100 + SQL_CODE_YEAR_TO_MONTH)
#SQL_INTERVAL_DAY_TO_HOUR = (100 + SQL_CODE_DAY_TO_HOUR)
#SQL_INTERVAL_DAY_TO_MINUTE = (100 + SQL_CODE_DAY_TO_MINUTE)
#SQL_INTERVAL_DAY_TO_SECOND = (100 + SQL_CODE_DAY_TO_SECOND)
#SQL_INTERVAL_HOUR_TO_MINUTE = (100 + SQL_CODE_HOUR_TO_MINUTE)
#SQL_INTERVAL_HOUR_TO_SECOND = (100 + SQL_CODE_HOUR_TO_SECOND)
#SQL_INTERVAL_MINUTE_TO_SECOND = (100 + SQL_CODE_MINUTE_TO_SECOND)
#SQL_INTERVAL_YEAR = (-80)
#SQL_INTERVAL_MONTH = (-81)
#SQL_INTERVAL_YEAR_TO_MONTH = (-82)
#SQL_INTERVAL_DAY = (-83)
#SQL_INTERVAL_HOUR = (-84)
#SQL_INTERVAL_MINUTE = (-85)
#SQL_INTERVAL_SECOND = (-86)
#SQL_INTERVAL_DAY_TO_HOUR = (-87)
#SQL_INTERVAL_DAY_TO_MINUTE = (-88)
#SQL_INTERVAL_DAY_TO_SECOND = (-89)
#SQL_INTERVAL_HOUR_TO_MINUTE = (-90)
#SQL_INTERVAL_HOUR_TO_SECOND = (-91)
#SQL_INTERVAL_MINUTE_TO_SECOND = (-92)
#SQL_UNICODE = (-95)
#SQL_UNICODE_VARCHAR = (-96)
#SQL_UNICODE_LONGVARCHAR = (-97)
#SQL_UNICODE_CHAR = SQL_UNICODE
#SQL_TYPE_DRIVER_START = SQL_INTERVAL_YEAR
#SQL_TYPE_DRIVER_END = SQL_UNICODE_LONGVARCHAR
#SQL_C_DEFAULT = 99
#SQL_SIGNED_OFFSET = (-20)
#SQL_UNSIGNED_OFFSET = (-22)
#SQL_C_DATE = SQL_DATE
#SQL_C_TIME = SQL_TIME
#SQL_C_TIMESTAMP = SQL_TIMESTAMP
#SQL_C_INTERVAL_YEAR = SQL_INTERVAL_YEAR
#SQL_C_INTERVAL_MONTH = SQL_INTERVAL_MONTH
#SQL_C_INTERVAL_DAY = SQL_INTERVAL_DAY
#SQL_C_INTERVAL_HOUR = SQL_INTERVAL_HOUR
#SQL_C_INTERVAL_MINUTE = SQL_INTERVAL_MINUTE
#SQL_C_INTERVAL_SECOND = SQL_INTERVAL_SECOND
#SQL_C_INTERVAL_YEAR_TO_MONTH = SQL_INTERVAL_YEAR_TO_MONTH
#SQL_C_INTERVAL_DAY_TO_HOUR = SQL_INTERVAL_DAY_TO_HOUR
#SQL_C_INTERVAL_DAY_TO_MINUTE = SQL_INTERVAL_DAY_TO_MINUTE
#SQL_C_INTERVAL_DAY_TO_SECOND = SQL_INTERVAL_DAY_TO_SECOND
#SQL_C_INTERVAL_HOUR_TO_MINUTE = SQL_INTERVAL_HOUR_TO_MINUTE
#SQL_C_INTERVAL_HOUR_TO_SECOND = SQL_INTERVAL_HOUR_TO_SECOND
#SQL_C_INTERVAL_MINUTE_TO_SECOND = SQL_INTERVAL_MINUTE_TO_SECOND
#SQL_C_BINARY = SQL_BINARY
#SQL_C_BIT = SQL_BIT
#SQL_C_SBIGINT = (SQL_BIGINT+SQL_SIGNED_OFFSET)
#SQL_C_UBIGINT = (SQL_BIGINT+SQL_UNSIGNED_OFFSET)
#SQL_C_TINYINT = SQL_TINYINT
#SQL_C_STINYINT = (SQL_TINYINT+SQL_SIGNED_OFFSET)
#SQL_C_UTINYINT = (SQL_TINYINT+SQL_UNSIGNED_OFFSET)
#SQL_C_BOOKMARK = SQL_C_UBIGINT
#SQL_C_GUID = SQL_GUID
#SQL_TYPE_NULL = 0
#SQL_TYPE_MIN = SQL_BIT
#SQL_C_VARBOOKMARK = SQL_C_BINARY
#SQL_NO_ROW_NUMBER = (-1)
#SQL_NO_COLUMN_NUMBER = (-1)
#SQL_ROW_NUMBER_UNKNOWN = (-2)
#SQL_COLUMN_NUMBER_UNKNOWN = (-2)
#SQL_DEFAULT_PARAM = (-5)
#SQL_IGNORE = (-6)
#SQL_COLUMN_IGNORE = SQL_IGNORE
#SQL_LEN_DATA_AT_EXEC_OFFSET = (-100)
#def SQL_LEN_DATA_AT_EXEC(length): return (-(length)+SQL_LEN_DATA_AT_EXEC_OFFSET)
#
#SQL_LEN_BINARY_ATTR_OFFSET = (-100)
#def SQL_LEN_BINARY_ATTR(length): return (-(length)+SQL_LEN_BINARY_ATTR_OFFSET)
#
#SQL_SETPARAM_VALUE_MAX = (-1L)
#SQL_COLUMN_COUNT = 0
#SQL_COLUMN_NAME = 1
#SQL_COLUMN_TYPE = 2
#SQL_COLUMN_LENGTH = 3
#SQL_COLUMN_PRECISION = 4
#SQL_COLUMN_SCALE = 5
#SQL_COLUMN_DISPLAY_SIZE = 6
#SQL_COLUMN_NULLABLE = 7
#SQL_COLUMN_UNSIGNED = 8
#SQL_COLUMN_MONEY = 9
#SQL_COLUMN_UPDATABLE = 10
#SQL_COLUMN_AUTO_INCREMENT = 11
#SQL_COLUMN_CASE_SENSITIVE = 12
#SQL_COLUMN_SEARCHABLE = 13
#SQL_COLUMN_TYPE_NAME = 14
#SQL_COLUMN_TABLE_NAME = 15
#SQL_COLUMN_OWNER_NAME = 16
#SQL_COLUMN_QUALIFIER_NAME = 17
#SQL_COLUMN_LABEL = 18
#SQL_COLATT_OPT_MAX = SQL_COLUMN_LABEL
#SQL_COLUMN_DRIVER_START = 1000
#SQL_COLATT_OPT_MIN = SQL_COLUMN_COUNT
#SQL_ATTR_READONLY = 0
#SQL_ATTR_WRITE = 1
#SQL_ATTR_READWRITE_UNKNOWN = 2
#SQL_UNSEARCHABLE = 0
#SQL_LIKE_ONLY = 1
#SQL_ALL_EXCEPT_LIKE = 2
#SQL_SEARCHABLE = 3
#SQL_PRED_SEARCHABLE = SQL_SEARCHABLE
#SQL_NO_TOTAL = (-4)
#SQL_API_SQLALLOCHANDLESTD = 73
#SQL_API_SQLBULKOPERATIONS = 24
#SQL_API_SQLBINDPARAMETER = 72
#SQL_API_SQLBROWSECONNECT = 55
#SQL_API_SQLCOLATTRIBUTES = 6
#SQL_API_SQLCOLUMNPRIVILEGES = 56
#SQL_API_SQLDESCRIBEPARAM = 58
#SQL_API_SQLDRIVERCONNECT = 41
#SQL_API_SQLDRIVERS = 71
#SQL_API_SQLEXTENDEDFETCH = 59
#SQL_API_SQLFOREIGNKEYS = 60
#SQL_API_SQLMORERESULTS = 61
#SQL_API_SQLNATIVESQL = 62
#SQL_API_SQLNUMPARAMS = 63
#SQL_API_SQLPARAMOPTIONS = 64
#SQL_API_SQLPRIMARYKEYS = 65
#SQL_API_SQLPROCEDURECOLUMNS = 66
#SQL_API_SQLPROCEDURES = 67
#SQL_API_SQLSETPOS = 68
#SQL_API_SQLSETSCROLLOPTIONS = 69
#SQL_API_SQLTABLEPRIVILEGES = 70
#SQL_EXT_API_LAST = SQL_API_SQLBINDPARAMETER
#SQL_NUM_FUNCTIONS = 23
#SQL_EXT_API_START = 40
#SQL_NUM_EXTENSIONS = (SQL_EXT_API_LAST-SQL_EXT_API_START+1)
#SQL_API_ALL_FUNCTIONS = 0
#SQL_API_LOADBYORDINAL = 199
#SQL_API_ODBC3_ALL_FUNCTIONS = 999
#SQL_API_ODBC3_ALL_FUNCTIONS_SIZE = 250
#SQL_INFO_FIRST = 0
#SQL_ACTIVE_CONNECTIONS = 0
#SQL_ACTIVE_STATEMENTS = 1
#SQL_DRIVER_HDBC = 3
#SQL_DRIVER_HENV = 4
#SQL_DRIVER_HSTMT = 5
#SQL_DRIVER_NAME = 6
#SQL_DRIVER_VER = 7
#SQL_ODBC_API_CONFORMANCE = 9
#SQL_ODBC_VER = 10
#SQL_ROW_UPDATES = 11
#SQL_ODBC_SAG_CLI_CONFORMANCE = 12
#SQL_ODBC_SQL_CONFORMANCE = 15
#SQL_PROCEDURES = 21
#SQL_CONCAT_NULL_BEHAVIOR = 22
#SQL_CURSOR_ROLLBACK_BEHAVIOR = 24
#SQL_EXPRESSIONS_IN_ORDERBY = 27
#SQL_MAX_OWNER_NAME_LEN = 32
#SQL_MAX_PROCEDURE_NAME_LEN = 33
#SQL_MAX_QUALIFIER_NAME_LEN = 34
#SQL_MULT_RESULT_SETS = 36
#SQL_MULTIPLE_ACTIVE_TXN = 37
#SQL_OUTER_JOINS = 38
#SQL_OWNER_TERM = 39
#SQL_PROCEDURE_TERM = 40
#SQL_QUALIFIER_NAME_SEPARATOR = 41
#SQL_QUALIFIER_TERM = 42
#SQL_SCROLL_OPTIONS = 44
#SQL_TABLE_TERM = 45
#SQL_CONVERT_FUNCTIONS = 48
#SQL_NUMERIC_FUNCTIONS = 49
#SQL_STRING_FUNCTIONS = 50
#SQL_SYSTEM_FUNCTIONS = 51
#SQL_TIMEDATE_FUNCTIONS = 52
#SQL_CONVERT_BIGINT = 53
#SQL_CONVERT_BINARY = 54
#SQL_CONVERT_BIT = 55
#SQL_CONVERT_CHAR = 56
#SQL_CONVERT_DATE = 57
#SQL_CONVERT_DECIMAL = 58
#SQL_CONVERT_DOUBLE = 59
#SQL_CONVERT_FLOAT = 60
#SQL_CONVERT_INTEGER = 61
#SQL_CONVERT_LONGVARCHAR = 62
#SQL_CONVERT_NUMERIC = 63
#SQL_CONVERT_REAL = 64
#SQL_CONVERT_SMALLINT = 65
#SQL_CONVERT_TIME = 66
#SQL_CONVERT_TIMESTAMP = 67
#SQL_CONVERT_TINYINT = 68
#SQL_CONVERT_VARBINARY = 69
#SQL_CONVERT_VARCHAR = 70
#SQL_CONVERT_LONGVARBINARY = 71
#SQL_CONVERT_GUID = 173
#SQL_ODBC_SQL_OPT_IEF = 73
#SQL_CORRELATION_NAME = 74
#SQL_NON_NULLABLE_COLUMNS = 75
#SQL_DRIVER_HLIB = 76
#SQL_DRIVER_ODBC_VER = 77
#SQL_LOCK_TYPES = 78
#SQL_POS_OPERATIONS = 79
#SQL_POSITIONED_STATEMENTS = 80
#SQL_BOOKMARK_PERSISTENCE = 82
#SQL_STATIC_SENSITIVITY = 83
#SQL_FILE_USAGE = 84
#SQL_COLUMN_ALIAS = 87
#SQL_GROUP_BY = 88
#SQL_KEYWORDS = 89
#SQL_OWNER_USAGE = 91
#SQL_QUALIFIER_USAGE = 92
#SQL_QUOTED_IDENTIFIER_CASE = 93
#SQL_SUBQUERIES = 95
#SQL_UNION = 96
#SQL_MAX_ROW_SIZE_INCLUDES_LONG = 103
#SQL_MAX_CHAR_LITERAL_LEN = 108
#SQL_TIMEDATE_ADD_INTERVALS = 109
#SQL_TIMEDATE_DIFF_INTERVALS = 110
#SQL_NEED_LONG_DATA_LEN = 111
#SQL_MAX_BINARY_LITERAL_LEN = 112
#SQL_LIKE_ESCAPE_CLAUSE = 113
#SQL_QUALIFIER_LOCATION = 114
#SQL_OJ_CAPABILITIES = 65003
#SQL_INFO_LAST = SQL_QUALIFIER_LOCATION
#SQL_INFO_DRIVER_START = 1000
#SQL_ACTIVE_ENVIRONMENTS = 116
#SQL_ALTER_DOMAIN = 117
#SQL_SQL_CONFORMANCE = 118
#SQL_DATETIME_LITERALS = 119
#SQL_ASYNC_MODE = 10021
#SQL_BATCH_ROW_COUNT = 120
#SQL_BATCH_SUPPORT = 121
#SQL_CATALOG_LOCATION = SQL_QUALIFIER_LOCATION
#SQL_CATALOG_NAME_SEPARATOR = SQL_QUALIFIER_NAME_SEPARATOR
#SQL_CATALOG_TERM = SQL_QUALIFIER_TERM
#SQL_CATALOG_USAGE = SQL_QUALIFIER_USAGE
#SQL_CONVERT_WCHAR = 122
#SQL_CONVERT_INTERVAL_DAY_TIME = 123
#SQL_CONVERT_INTERVAL_YEAR_MONTH = 124
#SQL_CONVERT_WLONGVARCHAR = 125
#SQL_CONVERT_WVARCHAR = 126
#SQL_CREATE_ASSERTION = 127
#SQL_CREATE_CHARACTER_SET = 128
#SQL_CREATE_COLLATION = 129
#SQL_CREATE_DOMAIN = 130
#SQL_CREATE_SCHEMA = 131
#SQL_CREATE_TABLE = 132
#SQL_CREATE_TRANSLATION = 133
#SQL_CREATE_VIEW = 134
#SQL_DRIVER_HDESC = 135
#SQL_DROP_ASSERTION = 136
#SQL_DROP_CHARACTER_SET = 137
#SQL_DROP_COLLATION = 138
#SQL_DROP_DOMAIN = 139
#SQL_DROP_SCHEMA = 140
#SQL_DROP_TABLE = 141
#SQL_DROP_TRANSLATION = 142
#SQL_DROP_VIEW = 143
#SQL_DYNAMIC_CURSOR_ATTRIBUTES1 = 144
#SQL_DYNAMIC_CURSOR_ATTRIBUTES2 = 145
#SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1 = 146
#SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2 = 147
#SQL_INDEX_KEYWORDS = 148
#SQL_INFO_SCHEMA_VIEWS = 149
#SQL_KEYSET_CURSOR_ATTRIBUTES1 = 150
#SQL_KEYSET_CURSOR_ATTRIBUTES2 = 151
#SQL_MAX_ASYNC_CONCURRENT_STATEMENTS = 10022
#SQL_ODBC_INTERFACE_CONFORMANCE = 152
#SQL_PARAM_ARRAY_ROW_COUNTS = 153
#SQL_PARAM_ARRAY_SELECTS = 154
#SQL_SCHEMA_TERM = SQL_OWNER_TERM
#SQL_SCHEMA_USAGE = SQL_OWNER_USAGE
#SQL_SQL92_DATETIME_FUNCTIONS = 155
#SQL_SQL92_FOREIGN_KEY_DELETE_RULE = 156
#SQL_SQL92_FOREIGN_KEY_UPDATE_RULE = 157
#SQL_SQL92_GRANT = 158
#SQL_SQL92_NUMERIC_VALUE_FUNCTIONS = 159
#SQL_SQL92_PREDICATES = 160
#SQL_SQL92_RELATIONAL_JOIN_OPERATORS = 161
#SQL_SQL92_REVOKE = 162
#SQL_SQL92_ROW_VALUE_CONSTRUCTOR = 163
#SQL_SQL92_STRING_FUNCTIONS = 164
#SQL_SQL92_VALUE_EXPRESSIONS = 165
#SQL_STANDARD_CLI_CONFORMANCE = 166
#SQL_STATIC_CURSOR_ATTRIBUTES1 = 167
#SQL_STATIC_CURSOR_ATTRIBUTES2 = 168
#SQL_AGGREGATE_FUNCTIONS = 169
#SQL_DDL_INDEX = 170
#SQL_DM_VER = 171
#SQL_INSERT_STATEMENT = 172
#SQL_UNION_STATEMENT = SQL_UNION
#SQL_DTC_TRANSITION_COST = 1750
#SQL_AT_ADD_COLUMN_SINGLE = 0x00000020L
#SQL_AT_ADD_COLUMN_DEFAULT = 0x00000040L
#SQL_AT_ADD_COLUMN_COLLATION = 0x00000080L
#SQL_AT_SET_COLUMN_DEFAULT = 0x00000100L
#SQL_AT_DROP_COLUMN_DEFAULT = 0x00000200L
#SQL_AT_DROP_COLUMN_CASCADE = 0x00000400L
#SQL_AT_DROP_COLUMN_RESTRICT = 0x00000800L
#SQL_AT_ADD_TABLE_CONSTRAINT = 0x00001000L
#SQL_AT_DROP_TABLE_CONSTRAINT_CASCADE = 0x00002000L
#SQL_AT_DROP_TABLE_CONSTRAINT_RESTRICT = 0x00004000L
#SQL_AT_CONSTRAINT_NAME_DEFINITION = 0x00008000L
#SQL_AT_CONSTRAINT_INITIALLY_DEFERRED = 0x00010000L
#SQL_AT_CONSTRAINT_INITIALLY_IMMEDIATE = 0x00020000L
#SQL_AT_CONSTRAINT_DEFERRABLE = 0x00040000L
#SQL_AT_CONSTRAINT_NON_DEFERRABLE = 0x00080000L
#SQL_CVT_CHAR = 0x00000001L
#SQL_CVT_NUMERIC = 0x00000002L
#SQL_CVT_DECIMAL = 0x00000004L
#SQL_CVT_INTEGER = 0x00000008L
#SQL_CVT_SMALLINT = 0x00000010L
#SQL_CVT_FLOAT = 0x00000020L
#SQL_CVT_REAL = 0x00000040L
#SQL_CVT_DOUBLE = 0x00000080L
#SQL_CVT_VARCHAR = 0x00000100L
#SQL_CVT_LONGVARCHAR = 0x00000200L
#SQL_CVT_BINARY = 0x00000400L
#SQL_CVT_VARBINARY = 0x00000800L
#SQL_CVT_BIT = 0x00001000L
#SQL_CVT_TINYINT = 0x00002000L
#SQL_CVT_BIGINT = 0x00004000L
#SQL_CVT_DATE = 0x00008000L
#SQL_CVT_TIME = 0x00010000L
#SQL_CVT_TIMESTAMP = 0x00020000L
#SQL_CVT_LONGVARBINARY = 0x00040000L
#SQL_CVT_INTERVAL_YEAR_MONTH = 0x00080000L
#SQL_CVT_INTERVAL_DAY_TIME = 0x00100000L
#SQL_CVT_WCHAR = 0x00200000L
#SQL_CVT_WLONGVARCHAR = 0x00400000L
#SQL_CVT_WVARCHAR = 0x00800000L
#SQL_CVT_GUID = 0x01000000L
#SQL_FN_CVT_CONVERT = 0x00000001L
#SQL_FN_CVT_CAST = 0x00000002L
#SQL_FN_STR_CONCAT = 0x00000001L
#SQL_FN_STR_INSERT = 0x00000002L
#SQL_FN_STR_LEFT = 0x00000004L
#SQL_FN_STR_LTRIM = 0x00000008L
#SQL_FN_STR_LENGTH = 0x00000010L
#SQL_FN_STR_LOCATE = 0x00000020L
#SQL_FN_STR_LCASE = 0x00000040L
#SQL_FN_STR_REPEAT = 0x00000080L
#SQL_FN_STR_REPLACE = 0x00000100L
#SQL_FN_STR_RIGHT = 0x00000200L
#SQL_FN_STR_RTRIM = 0x00000400L
#SQL_FN_STR_SUBSTRING = 0x00000800L
#SQL_FN_STR_UCASE = 0x00001000L
#SQL_FN_STR_ASCII = 0x00002000L
#SQL_FN_STR_CHAR = 0x00004000L
#SQL_FN_STR_DIFFERENCE = 0x00008000L
#SQL_FN_STR_LOCATE_2 = 0x00010000L
#SQL_FN_STR_SOUNDEX = 0x00020000L
#SQL_FN_STR_SPACE = 0x00040000L
#SQL_FN_STR_BIT_LENGTH = 0x00080000L
#SQL_FN_STR_CHAR_LENGTH = 0x00100000L
#SQL_FN_STR_CHARACTER_LENGTH = 0x00200000L
#SQL_FN_STR_OCTET_LENGTH = 0x00400000L
#SQL_FN_STR_POSITION = 0x00800000L
#SQL_SSF_CONVERT = 0x00000001L
#SQL_SSF_LOWER = 0x00000002L
#SQL_SSF_UPPER = 0x00000004L
#SQL_SSF_SUBSTRING = 0x00000008L
#SQL_SSF_TRANSLATE = 0x00000010L
#SQL_SSF_TRIM_BOTH = 0x00000020L
#SQL_SSF_TRIM_LEADING = 0x00000040L
#SQL_SSF_TRIM_TRAILING = 0x00000080L
#SQL_FN_NUM_ABS = 0x00000001L
#SQL_FN_NUM_ACOS = 0x00000002L
#SQL_FN_NUM_ASIN = 0x00000004L
#SQL_FN_NUM_ATAN = 0x00000008L
#SQL_FN_NUM_ATAN2 = 0x00000010L
#SQL_FN_NUM_CEILING = 0x00000020L
#SQL_FN_NUM_COS = 0x00000040L
#SQL_FN_NUM_COT = 0x00000080L
#SQL_FN_NUM_EXP = 0x00000100L
#SQL_FN_NUM_FLOOR = 0x00000200L
#SQL_FN_NUM_LOG = 0x00000400L
#SQL_FN_NUM_MOD = 0x00000800L
#SQL_FN_NUM_SIGN = 0x00001000L
#SQL_FN_NUM_SIN = 0x00002000L
#SQL_FN_NUM_SQRT = 0x00004000L
#SQL_FN_NUM_TAN = 0x00008000L
#SQL_FN_NUM_PI = 0x00010000L
#SQL_FN_NUM_RAND = 0x00020000L
#SQL_FN_NUM_DEGREES = 0x00040000L
#SQL_FN_NUM_LOG10 = 0x00080000L
#SQL_FN_NUM_POWER = 0x00100000L
#SQL_FN_NUM_RADIANS = 0x00200000L
#SQL_FN_NUM_ROUND = 0x00400000L
#SQL_FN_NUM_TRUNCATE = 0x00800000L
#SQL_SNVF_BIT_LENGTH = 0x00000001L
#SQL_SNVF_CHAR_LENGTH = 0x00000002L
#SQL_SNVF_CHARACTER_LENGTH = 0x00000004L
#SQL_SNVF_EXTRACT = 0x00000008L
#SQL_SNVF_OCTET_LENGTH = 0x00000010L
#SQL_SNVF_POSITION = 0x00000020L
#SQL_FN_TD_NOW = 0x00000001L
#SQL_FN_TD_CURDATE = 0x00000002L
#SQL_FN_TD_DAYOFMONTH = 0x00000004L
#SQL_FN_TD_DAYOFWEEK = 0x00000008L
#SQL_FN_TD_DAYOFYEAR = 0x00000010L
#SQL_FN_TD_MONTH = 0x00000020L
#SQL_FN_TD_QUARTER = 0x00000040L
#SQL_FN_TD_WEEK = 0x00000080L
#SQL_FN_TD_YEAR = 0x00000100L
#SQL_FN_TD_CURTIME = 0x00000200L
#SQL_FN_TD_HOUR = 0x00000400L
#SQL_FN_TD_MINUTE = 0x00000800L
#SQL_FN_TD_SECOND = 0x00001000L
#SQL_FN_TD_TIMESTAMPADD = 0x00002000L
#SQL_FN_TD_TIMESTAMPDIFF = 0x00004000L
#SQL_FN_TD_DAYNAME = 0x00008000L
#SQL_FN_TD_MONTHNAME = 0x00010000L
#SQL_FN_TD_CURRENT_DATE = 0x00020000L
#SQL_FN_TD_CURRENT_TIME = 0x00040000L
#SQL_FN_TD_CURRENT_TIMESTAMP = 0x00080000L
#SQL_FN_TD_EXTRACT = 0x00100000L
#SQL_SDF_CURRENT_DATE = 0x00000001L
#SQL_SDF_CURRENT_TIME = 0x00000002L
#SQL_SDF_CURRENT_TIMESTAMP = 0x00000004L
#SQL_FN_SYS_USERNAME = 0x00000001L
#SQL_FN_SYS_DBNAME = 0x00000002L
#SQL_FN_SYS_IFNULL = 0x00000004L
#SQL_FN_TSI_FRAC_SECOND = 0x00000001L
#SQL_FN_TSI_SECOND = 0x00000002L
#SQL_FN_TSI_MINUTE = 0x00000004L
#SQL_FN_TSI_HOUR = 0x00000008L
#SQL_FN_TSI_DAY = 0x00000010L
#SQL_FN_TSI_WEEK = 0x00000020L
#SQL_FN_TSI_MONTH = 0x00000040L
#SQL_FN_TSI_QUARTER = 0x00000080L
#SQL_FN_TSI_YEAR = 0x00000100L
#SQL_CA1_NEXT = 0x00000001L
#SQL_CA1_ABSOLUTE = 0x00000002L
#SQL_CA1_RELATIVE = 0x00000004L
#SQL_CA1_BOOKMARK = 0x00000008L
#SQL_CA1_LOCK_NO_CHANGE = 0x00000040L
#SQL_CA1_LOCK_EXCLUSIVE = 0x00000080L
#SQL_CA1_LOCK_UNLOCK = 0x00000100L
#SQL_CA1_POS_POSITION = 0x00000200L
#SQL_CA1_POS_UPDATE = 0x00000400L
#SQL_CA1_POS_DELETE = 0x00000800L
#SQL_CA1_POS_REFRESH = 0x00001000L
#SQL_CA1_POSITIONED_UPDATE = 0x00002000L
#SQL_CA1_POSITIONED_DELETE = 0x00004000L
#SQL_CA1_SELECT_FOR_UPDATE = 0x00008000L
#SQL_CA1_BULK_ADD = 0x00010000L
#SQL_CA1_BULK_UPDATE_BY_BOOKMARK = 0x00020000L
#SQL_CA1_BULK_DELETE_BY_BOOKMARK = 0x00040000L
#SQL_CA1_BULK_FETCH_BY_BOOKMARK = 0x00080000L
#SQL_CA2_READ_ONLY_CONCURRENCY = 0x00000001L
#SQL_CA2_LOCK_CONCURRENCY = 0x00000002L
#SQL_CA2_OPT_ROWVER_CONCURRENCY = 0x00000004L
#SQL_CA2_OPT_VALUES_CONCURRENCY = 0x00000008L
#SQL_CA2_SENSITIVITY_ADDITIONS = 0x00000010L
#SQL_CA2_SENSITIVITY_DELETIONS = 0x00000020L
#SQL_CA2_SENSITIVITY_UPDATES = 0x00000040L
#SQL_CA2_MAX_ROWS_SELECT = 0x00000080L
#SQL_CA2_MAX_ROWS_INSERT = 0x00000100L
#SQL_CA2_MAX_ROWS_DELETE = 0x00000200L
#SQL_CA2_MAX_ROWS_UPDATE = 0x00000400L
#SQL_CA2_MAX_ROWS_CATALOG = 0x00000800L
#SQL_CA2_MAX_ROWS_AFFECTS_ALL = (SQL_CA2_MAX_ROWS_SELECT | SQL_CA2_MAX_ROWS_INSERT | SQL_CA2_MAX_ROWS_DELETE | SQL_CA2_MAX_ROWS_UPDATE | SQL_CA2_MAX_ROWS_CATALOG)
#SQL_CA2_CRC_EXACT = 0x00001000L
#SQL_CA2_CRC_APPROXIMATE = 0x00002000L
#SQL_CA2_SIMULATE_NON_UNIQUE = 0x00004000L
#SQL_CA2_SIMULATE_TRY_UNIQUE = 0x00008000L
#SQL_CA2_SIMULATE_UNIQUE = 0x00010000L
#SQL_OAC_NONE = 0x0000
#SQL_OAC_LEVEL1 = 0x0001
#SQL_OAC_LEVEL2 = 0x0002
#SQL_OSCC_NOT_COMPLIANT = 0x0000
#SQL_OSCC_COMPLIANT = 0x0001
#SQL_OSC_MINIMUM = 0x0000
#SQL_OSC_CORE = 0x0001
#SQL_OSC_EXTENDED = 0x0002
#SQL_CB_NULL = 0x0000
#SQL_CB_NON_NULL = 0x0001
#SQL_SO_FORWARD_ONLY = 0x00000001L
#SQL_SO_KEYSET_DRIVEN = 0x00000002L
#SQL_SO_DYNAMIC = 0x00000004L
#SQL_SO_MIXED = 0x00000008L
#SQL_SO_STATIC = 0x00000010L
#SQL_FD_FETCH_RESUME = 0x00000040L
#SQL_FD_FETCH_BOOKMARK = 0x00000080L
#SQL_TXN_VERSIONING = 0x00000010L
#SQL_CN_NONE = 0x0000
#SQL_CN_DIFFERENT = 0x0001
#SQL_CN_ANY = 0x0002
#SQL_NNC_NULL = 0x0000
#SQL_NNC_NON_NULL = 0x0001
#SQL_NC_START = 0x0002
#SQL_NC_END = 0x0004
#SQL_FILE_NOT_SUPPORTED = 0x0000
#SQL_FILE_TABLE = 0x0001
#SQL_FILE_QUALIFIER = 0x0002
#SQL_FILE_CATALOG = SQL_FILE_QUALIFIER
#SQL_GD_BLOCK = 0x00000004L
#SQL_GD_BOUND = 0x00000008L
#SQL_PS_POSITIONED_DELETE = 0x00000001L
#SQL_PS_POSITIONED_UPDATE = 0x00000002L
#SQL_PS_SELECT_FOR_UPDATE = 0x00000004L
#SQL_GB_NOT_SUPPORTED = 0x0000
#SQL_GB_GROUP_BY_EQUALS_SELECT = 0x0001
#SQL_GB_GROUP_BY_CONTAINS_SELECT = 0x0002
#SQL_GB_NO_RELATION = 0x0003
#SQL_GB_COLLATE = 0x0004
#SQL_OU_DML_STATEMENTS = 0x00000001L
#SQL_OU_PROCEDURE_INVOCATION = 0x00000002L
#SQL_OU_TABLE_DEFINITION = 0x00000004L
#SQL_OU_INDEX_DEFINITION = 0x00000008L
#SQL_OU_PRIVILEGE_DEFINITION = 0x00000010L
#SQL_SU_DML_STATEMENTS = SQL_OU_DML_STATEMENTS
#SQL_SU_PROCEDURE_INVOCATION = SQL_OU_PROCEDURE_INVOCATION
#SQL_SU_TABLE_DEFINITION = SQL_OU_TABLE_DEFINITION
#SQL_SU_INDEX_DEFINITION = SQL_OU_INDEX_DEFINITION
#SQL_SU_PRIVILEGE_DEFINITION = SQL_OU_PRIVILEGE_DEFINITION
#SQL_QU_DML_STATEMENTS = 0x00000001L
#SQL_QU_PROCEDURE_INVOCATION = 0x00000002L
#SQL_QU_TABLE_DEFINITION = 0x00000004L
#SQL_QU_INDEX_DEFINITION = 0x00000008L
#SQL_QU_PRIVILEGE_DEFINITION = 0x00000010L
#SQL_CU_DML_STATEMENTS = SQL_QU_DML_STATEMENTS
#SQL_CU_PROCEDURE_INVOCATION = SQL_QU_PROCEDURE_INVOCATION
#SQL_CU_TABLE_DEFINITION = SQL_QU_TABLE_DEFINITION
#SQL_CU_INDEX_DEFINITION = SQL_QU_INDEX_DEFINITION
#SQL_CU_PRIVILEGE_DEFINITION = SQL_QU_PRIVILEGE_DEFINITION
#SQL_SQ_COMPARISON = 0x00000001L
#SQL_SQ_EXISTS = 0x00000002L
#SQL_SQ_IN = 0x00000004L
#SQL_SQ_QUANTIFIED = 0x00000008L
#SQL_SQ_CORRELATED_SUBQUERIES = 0x00000010L
#SQL_U_UNION = 0x00000001L
#SQL_U_UNION_ALL = 0x00000002L
#SQL_BP_CLOSE = 0x00000001L
#SQL_BP_DELETE = 0x00000002L
#SQL_BP_DROP = 0x00000004L
#SQL_BP_TRANSACTION = 0x00000008L
#SQL_BP_UPDATE = 0x00000010L
#SQL_BP_OTHER_HSTMT = 0x00000020L
#SQL_BP_SCROLL = 0x00000040L
#SQL_SS_ADDITIONS = 0x00000001L
#SQL_SS_DELETIONS = 0x00000002L
#SQL_SS_UPDATES = 0x00000004L
#SQL_CV_CREATE_VIEW = 0x00000001L
#SQL_CV_CHECK_OPTION = 0x00000002L
#SQL_CV_CASCADED = 0x00000004L
#SQL_CV_LOCAL = 0x00000008L
#SQL_LCK_NO_CHANGE = 0x00000001L
#SQL_LCK_EXCLUSIVE = 0x00000002L
#SQL_LCK_UNLOCK = 0x00000004L
#SQL_POS_POSITION = 0x00000001L
#SQL_POS_REFRESH = 0x00000002L
#SQL_POS_UPDATE = 0x00000004L
#SQL_POS_DELETE = 0x00000008L
#SQL_POS_ADD = 0x00000010L
#SQL_QL_START = 0x0001
#SQL_QL_END = 0x0002
#SQL_AF_AVG = 0x00000001L
#SQL_AF_COUNT = 0x00000002L
#SQL_AF_MAX = 0x00000004L
#SQL_AF_MIN = 0x00000008L
#SQL_AF_SUM = 0x00000010L
#SQL_AF_DISTINCT = 0x00000020L
#SQL_AF_ALL = 0x00000040L
#SQL_SC_SQL92_ENTRY = 0x00000001L
#SQL_SC_FIPS127_2_TRANSITIONAL = 0x00000002L
#SQL_SC_SQL92_INTERMEDIATE = 0x00000004L
#SQL_SC_SQL92_FULL = 0x00000008L
#SQL_DL_SQL92_DATE = 0x00000001L
#SQL_DL_SQL92_TIME = 0x00000002L
#SQL_DL_SQL92_TIMESTAMP = 0x00000004L
#SQL_DL_SQL92_INTERVAL_YEAR = 0x00000008L
#SQL_DL_SQL92_INTERVAL_MONTH = 0x00000010L
#SQL_DL_SQL92_INTERVAL_DAY = 0x00000020L
#SQL_DL_SQL92_INTERVAL_HOUR = 0x00000040L
#SQL_DL_SQL92_INTERVAL_MINUTE = 0x00000080L
#SQL_DL_SQL92_INTERVAL_SECOND = 0x00000100L
#SQL_DL_SQL92_INTERVAL_YEAR_TO_MONTH = 0x00000200L
#SQL_DL_SQL92_INTERVAL_DAY_TO_HOUR = 0x00000400L
#SQL_DL_SQL92_INTERVAL_DAY_TO_MINUTE = 0x00000800L
#SQL_DL_SQL92_INTERVAL_DAY_TO_SECOND = 0x00001000L
#SQL_DL_SQL92_INTERVAL_HOUR_TO_MINUTE = 0x00002000L
#SQL_DL_SQL92_INTERVAL_HOUR_TO_SECOND = 0x00004000L
#SQL_DL_SQL92_INTERVAL_MINUTE_TO_SECOND = 0x00008000L
#SQL_CL_START = SQL_QL_START
#SQL_CL_END = SQL_QL_END
#SQL_BRC_PROCEDURES = 0x0000001
#SQL_BRC_EXPLICIT = 0x0000002
#SQL_BRC_ROLLED_UP = 0x0000004
#SQL_BS_SELECT_EXPLICIT = 0x00000001L
#SQL_BS_ROW_COUNT_EXPLICIT = 0x00000002L
#SQL_BS_SELECT_PROC = 0x00000004L
#SQL_BS_ROW_COUNT_PROC = 0x00000008L
#SQL_PARC_BATCH = 1
#SQL_PARC_NO_BATCH = 2
#SQL_PAS_BATCH = 1
#SQL_PAS_NO_BATCH = 2
#SQL_PAS_NO_SELECT = 3
#SQL_IK_NONE = 0x00000000L
#SQL_IK_ASC = 0x00000001L
#SQL_IK_DESC = 0x00000002L
#SQL_IK_ALL = (SQL_IK_ASC | SQL_IK_DESC)
#SQL_ISV_ASSERTIONS = 0x00000001L
#SQL_ISV_CHARACTER_SETS = 0x00000002L
#SQL_ISV_CHECK_CONSTRAINTS = 0x00000004L
#SQL_ISV_COLLATIONS = 0x00000008L
#SQL_ISV_COLUMN_DOMAIN_USAGE = 0x00000010L
#SQL_ISV_COLUMN_PRIVILEGES = 0x00000020L
#SQL_ISV_COLUMNS = 0x00000040L
#SQL_ISV_CONSTRAINT_COLUMN_USAGE = 0x00000080L
#SQL_ISV_CONSTRAINT_TABLE_USAGE = 0x00000100L
#SQL_ISV_DOMAIN_CONSTRAINTS = 0x00000200L
#SQL_ISV_DOMAINS = 0x00000400L
#SQL_ISV_KEY_COLUMN_USAGE = 0x00000800L
#SQL_ISV_REFERENTIAL_CONSTRAINTS = 0x00001000L
#SQL_ISV_SCHEMATA = 0x00002000L
#SQL_ISV_SQL_LANGUAGES = 0x00004000L
#SQL_ISV_TABLE_CONSTRAINTS = 0x00008000L
#SQL_ISV_TABLE_PRIVILEGES = 0x00010000L
#SQL_ISV_TABLES = 0x00020000L
#SQL_ISV_TRANSLATIONS = 0x00040000L
#SQL_ISV_USAGE_PRIVILEGES = 0x00080000L
#SQL_ISV_VIEW_COLUMN_USAGE = 0x00100000L
#SQL_ISV_VIEW_TABLE_USAGE = 0x00200000L
#SQL_ISV_VIEWS = 0x00400000L
#SQL_AM_NONE = 0
#SQL_AM_CONNECTION = 1
#SQL_AM_STATEMENT = 2
#SQL_AD_CONSTRAINT_NAME_DEFINITION = 0x00000001L
#SQL_AD_ADD_DOMAIN_CONSTRAINT = 0x00000002L
#SQL_AD_DROP_DOMAIN_CONSTRAINT = 0x00000004L
#SQL_AD_ADD_DOMAIN_DEFAULT = 0x00000008L
#SQL_AD_DROP_DOMAIN_DEFAULT = 0x00000010L
#SQL_AD_ADD_CONSTRAINT_INITIALLY_DEFERRED = 0x00000020L
#SQL_AD_ADD_CONSTRAINT_INITIALLY_IMMEDIATE = 0x00000040L
#SQL_AD_ADD_CONSTRAINT_DEFERRABLE = 0x00000080L
#SQL_AD_ADD_CONSTRAINT_NON_DEFERRABLE = 0x00000100L
#SQL_CS_CREATE_SCHEMA = 0x00000001L
#SQL_CS_AUTHORIZATION = 0x00000002L
#SQL_CS_DEFAULT_CHARACTER_SET = 0x00000004L
#SQL_CTR_CREATE_TRANSLATION = 0x00000001L
#SQL_CA_CREATE_ASSERTION = 0x00000001L
#SQL_CA_CONSTRAINT_INITIALLY_DEFERRED = 0x00000010L
#SQL_CA_CONSTRAINT_INITIALLY_IMMEDIATE = 0x00000020L
#SQL_CA_CONSTRAINT_DEFERRABLE = 0x00000040L
#SQL_CA_CONSTRAINT_NON_DEFERRABLE = 0x00000080L
#SQL_CCS_CREATE_CHARACTER_SET = 0x00000001L
#SQL_CCS_COLLATE_CLAUSE = 0x00000002L
#SQL_CCS_LIMITED_COLLATION = 0x00000004L
#SQL_CCOL_CREATE_COLLATION = 0x00000001L
#SQL_CDO_CREATE_DOMAIN = 0x00000001L
#SQL_CDO_DEFAULT = 0x00000002L
#SQL_CDO_CONSTRAINT = 0x00000004L
#SQL_CDO_COLLATION = 0x00000008L
#SQL_CDO_CONSTRAINT_NAME_DEFINITION = 0x00000010L
#SQL_CDO_CONSTRAINT_INITIALLY_DEFERRED = 0x00000020L
#SQL_CDO_CONSTRAINT_INITIALLY_IMMEDIATE = 0x00000040L
#SQL_CDO_CONSTRAINT_DEFERRABLE = 0x00000080L
#SQL_CDO_CONSTRAINT_NON_DEFERRABLE = 0x00000100L
#SQL_CT_CREATE_TABLE = 0x00000001L
#SQL_CT_COMMIT_PRESERVE = 0x00000002L
#SQL_CT_COMMIT_DELETE = 0x00000004L
#SQL_CT_GLOBAL_TEMPORARY = 0x00000008L
#SQL_CT_LOCAL_TEMPORARY = 0x00000010L
#SQL_CT_CONSTRAINT_INITIALLY_DEFERRED = 0x00000020L
#SQL_CT_CONSTRAINT_INITIALLY_IMMEDIATE = 0x00000040L
#SQL_CT_CONSTRAINT_DEFERRABLE = 0x00000080L
#SQL_CT_CONSTRAINT_NON_DEFERRABLE = 0x00000100L
#SQL_CT_COLUMN_CONSTRAINT = 0x00000200L
#SQL_CT_COLUMN_DEFAULT = 0x00000400L
#SQL_CT_COLUMN_COLLATION = 0x00000800L
#SQL_CT_TABLE_CONSTRAINT = 0x00001000L
#SQL_CT_CONSTRAINT_NAME_DEFINITION = 0x00002000L
#SQL_DI_CREATE_INDEX = 0x00000001L
#SQL_DI_DROP_INDEX = 0x00000002L
#SQL_DC_DROP_COLLATION = 0x00000001L
#SQL_DD_DROP_DOMAIN = 0x00000001L
#SQL_DD_RESTRICT = 0x00000002L
#SQL_DD_CASCADE = 0x00000004L
#SQL_DS_DROP_SCHEMA = 0x00000001L
#SQL_DS_RESTRICT = 0x00000002L
#SQL_DS_CASCADE = 0x00000004L
#SQL_DCS_DROP_CHARACTER_SET = 0x00000001L
#SQL_DA_DROP_ASSERTION = 0x00000001L
#SQL_DT_DROP_TABLE = 0x00000001L
#SQL_DT_RESTRICT = 0x00000002L
#SQL_DT_CASCADE = 0x00000004L
#SQL_DTR_DROP_TRANSLATION = 0x00000001L
#SQL_DV_DROP_VIEW = 0x00000001L
#SQL_DV_RESTRICT = 0x00000002L
#SQL_DV_CASCADE = 0x00000004L
#SQL_IS_INSERT_LITERALS = 0x00000001L
#SQL_IS_INSERT_SEARCHED = 0x00000002L
#SQL_IS_SELECT_INTO = 0x00000004L
#SQL_SFKD_CASCADE = 0x00000001L
#SQL_SFKD_NO_ACTION = 0x00000002L
#SQL_SFKD_SET_DEFAULT = 0x00000004L
#SQL_SFKD_SET_NULL = 0x00000008L
#SQL_SFKU_CASCADE = 0x00000001L
#SQL_SFKU_NO_ACTION = 0x00000002L
#SQL_SFKU_SET_DEFAULT = 0x00000004L
#SQL_SFKU_SET_NULL = 0x00000008L
#SQL_SG_USAGE_ON_DOMAIN = 0x00000001L
#SQL_SG_USAGE_ON_CHARACTER_SET = 0x00000002L
#SQL_SG_USAGE_ON_COLLATION = 0x00000004L
#SQL_SG_USAGE_ON_TRANSLATION = 0x00000008L
#SQL_SG_WITH_GRANT_OPTION = 0x00000010L
#SQL_SG_DELETE_TABLE = 0x00000020L
#SQL_SG_INSERT_TABLE = 0x00000040L
#SQL_SG_INSERT_COLUMN = 0x00000080L
#SQL_SG_REFERENCES_TABLE = 0x00000100L
#SQL_SG_REFERENCES_COLUMN = 0x00000200L
#SQL_SG_SELECT_TABLE = 0x00000400L
#SQL_SG_UPDATE_TABLE = 0x00000800L
#SQL_SG_UPDATE_COLUMN = 0x00001000L
#SQL_SP_EXISTS = 0x00000001L
#SQL_SP_ISNOTNULL = 0x00000002L
#SQL_SP_ISNULL = 0x00000004L
#SQL_SP_MATCH_FULL = 0x00000008L
#SQL_SP_MATCH_PARTIAL = 0x00000010L
#SQL_SP_MATCH_UNIQUE_FULL = 0x00000020L
#SQL_SP_MATCH_UNIQUE_PARTIAL = 0x00000040L
#SQL_SP_OVERLAPS = 0x00000080L
#SQL_SP_UNIQUE = 0x00000100L
#SQL_SP_LIKE = 0x00000200L
#SQL_SP_IN = 0x00000400L
#SQL_SP_BETWEEN = 0x00000800L
#SQL_SP_COMPARISON = 0x00001000L
#SQL_SP_QUANTIFIED_COMPARISON = 0x00002000L
#SQL_SRJO_CORRESPONDING_CLAUSE = 0x00000001L
#SQL_SRJO_CROSS_JOIN = 0x00000002L
#SQL_SRJO_EXCEPT_JOIN = 0x00000004L
#SQL_SRJO_FULL_OUTER_JOIN = 0x00000008L
#SQL_SRJO_INNER_JOIN = 0x00000010L
#SQL_SRJO_INTERSECT_JOIN = 0x00000020L
#SQL_SRJO_LEFT_OUTER_JOIN = 0x00000040L
#SQL_SRJO_NATURAL_JOIN = 0x00000080L
#SQL_SRJO_RIGHT_OUTER_JOIN = 0x00000100L
#SQL_SRJO_UNION_JOIN = 0x00000200L
#SQL_SR_USAGE_ON_DOMAIN = 0x00000001L
#SQL_SR_USAGE_ON_CHARACTER_SET = 0x00000002L
#SQL_SR_USAGE_ON_COLLATION = 0x00000004L
#SQL_SR_USAGE_ON_TRANSLATION = 0x00000008L
#SQL_SR_GRANT_OPTION_FOR = 0x00000010L
#SQL_SR_CASCADE = 0x00000020L
#SQL_SR_RESTRICT = 0x00000040L
#SQL_SR_DELETE_TABLE = 0x00000080L
#SQL_SR_INSERT_TABLE = 0x00000100L
#SQL_SR_INSERT_COLUMN = 0x00000200L
#SQL_SR_REFERENCES_TABLE = 0x00000400L
#SQL_SR_REFERENCES_COLUMN = 0x00000800L
#SQL_SR_SELECT_TABLE = 0x00001000L
#SQL_SR_UPDATE_TABLE = 0x00002000L
#SQL_SR_UPDATE_COLUMN = 0x00004000L
#SQL_SRVC_VALUE_EXPRESSION = 0x00000001L
#SQL_SRVC_NULL = 0x00000002L
#SQL_SRVC_DEFAULT = 0x00000004L
#SQL_SRVC_ROW_SUBQUERY = 0x00000008L
#SQL_SVE_CASE = 0x00000001L
#SQL_SVE_CAST = 0x00000002L
#SQL_SVE_COALESCE = 0x00000004L
#SQL_SVE_NULLIF = 0x00000008L
#SQL_SCC_XOPEN_CLI_VERSION1 = 0x00000001L
#SQL_SCC_ISO92_CLI = 0x00000002L
#SQL_US_UNION = SQL_U_UNION
#SQL_US_UNION_ALL = SQL_U_UNION_ALL
#SQL_DTC_ENLIST_EXPENSIVE = 0x00000001L
#SQL_DTC_UNENLIST_EXPENSIVE = 0x00000002L
#SQL_FETCH_FIRST_USER = 31
#SQL_FETCH_FIRST_SYSTEM = 32
#SQL_ENTIRE_ROWSET = 0
#SQL_POSITION = 0
#SQL_REFRESH = 1
#SQL_UPDATE = 2
#SQL_DELETE = 3
#SQL_ADD = 4
#SQL_SETPOS_MAX_OPTION_VALUE = SQL_ADD
#SQL_UPDATE_BY_BOOKMARK = 5
#SQL_DELETE_BY_BOOKMARK = 6
#SQL_FETCH_BY_BOOKMARK = 7
#SQL_LOCK_NO_CHANGE = 0
#SQL_LOCK_EXCLUSIVE = 1
#SQL_LOCK_UNLOCK = 2
#SQL_SETPOS_MAX_LOCK_VALUE = SQL_LOCK_UNLOCK
#SQL_BEST_ROWID = 1
#SQL_ROWVER = 2
#SQL_PC_NOT_PSEUDO = 1
#SQL_QUICK = 0
#SQL_ENSURE = 1
SQL_TABLE_STAT = 0
#SQL_ALL_CATALOGS = "%"
#SQL_ALL_SCHEMAS = "%"
#SQL_ALL_TABLE_TYPES = "%"
#SQL_DRIVER_NOPROMPT = 0
#SQL_DRIVER_COMPLETE = 1
#SQL_DRIVER_PROMPT = 2
#SQL_DRIVER_COMPLETE_REQUIRED = 3
#SQL_FETCH_BOOKMARK = 8
#SQL_ROW_SUCCESS = 0
#SQL_ROW_DELETED = 1
#SQL_ROW_UPDATED = 2
#SQL_ROW_NOROW = 3
#SQL_ROW_ADDED = 4
#SQL_ROW_ERROR = 5
#SQL_ROW_SUCCESS_WITH_INFO = 6
#SQL_ROW_PROCEED = 0
#SQL_ROW_IGNORE = 1
#SQL_PARAM_SUCCESS = 0
#SQL_PARAM_SUCCESS_WITH_INFO = 6
#SQL_PARAM_ERROR = 5
#SQL_PARAM_UNUSED = 7
#SQL_PARAM_DIAG_UNAVAILABLE = 1
#SQL_PARAM_PROCEED = 0
#SQL_PARAM_IGNORE = 1
SQL_CASCADE = 0
SQL_RESTRICT = 1
SQL_SET_NULL = 2
SQL_NO_ACTION = 3
SQL_SET_DEFAULT = 4
#SQL_INITIALLY_DEFERRED = 5
#SQL_INITIALLY_IMMEDIATE = 6
#SQL_NOT_DEFERRABLE = 7
#SQL_PARAM_TYPE_UNKNOWN = 0
#SQL_PARAM_INPUT = 1
#SQL_PARAM_INPUT_OUTPUT = 2
#SQL_RESULT_COL = 3
#SQL_PARAM_OUTPUT = 4
#SQL_RETURN_VALUE = 5
#SQL_PT_UNKNOWN = 0
#SQL_PT_PROCEDURE = 1
#SQL_PT_FUNCTION = 2
#SQL_ODBC_KEYWORDS = "ABSOLUTE,ACTION,ADA,ADD,ALL,ALLOCATE,ALTER,AND,ANY,ARE,AS,"\
#"ASC,ASSERTION,AT,AUTHORIZATION,AVG,"\
#"BEGIN,BETWEEN,BIT,BIT_LENGTH,BOTH,BY,CASCADE,CASCADED,CASE,CAST,CATALOG,"\
#"CHAR,CHAR_LENGTH,CHARACTER,CHARACTER_LENGTH,CHECK,CLOSE,COALESCE,"\
#"COLLATE,COLLATION,COLUMN,COMMIT,CONNECT,CONNECTION,CONSTRAINT,"\
#"CONSTRAINTS,CONTINUE,CONVERT,CORRESPONDING,COUNT,CREATE,CROSS,CURRENT,"\
#"CURRENT_DATE,CURRENT_TIME,CURRENT_TIMESTAMP,CURRENT_USER,CURSOR,"\
#"DATE,DAY,DEALLOCATE,DEC,DECIMAL,DECLARE,DEFAULT,DEFERRABLE,"\
#"DEFERRED,DELETE,DESC,DESCRIBE,DESCRIPTOR,DIAGNOSTICS,DISCONNECT,"\
#"DISTINCT,DOMAIN,DOUBLE,DROP,"\
#"ELSE,END,END-EXEC,ESCAPE,EXCEPT,EXCEPTION,EXEC,EXECUTE,"\
#"EXISTS,EXTERNAL,EXTRACT,"\
#"FALSE,FETCH,FIRST,FLOAT,FOR,FOREIGN,FORTRAN,FOUND,FROM,FULL,"\
#"GET,GLOBAL,GO,GOTO,GRANT,GROUP,HAVING,HOUR,"\
#"IDENTITY,IMMEDIATE,IN,INCLUDE,INDEX,INDICATOR,INITIALLY,INNER,"\
#"INPUT,INSENSITIVE,INSERT,INT,INTEGER,INTERSECT,INTERVAL,INTO,IS,ISOLATION,"\
#"JOIN,KEY,LANGUAGE,LAST,LEADING,LEFT,LEVEL,LIKE,LOCAL,LOWER,"\
#"MATCH,MAX,MIN,MINUTE,MODULE,MONTH,"\
#"NAMES,NATIONAL,NATURAL,NCHAR,NEXT,NO,NONE,NOT,NULL,NULLIF,NUMERIC,"\
#"OCTET_LENGTH,OF,ON,ONLY,OPEN,OPTION,OR,ORDER,OUTER,OUTPUT,OVERLAPS,"\
#"PAD,PARTIAL,PASCAL,PLI,POSITION,PRECISION,PREPARE,PRESERVE,"\
#"PRIMARY,PRIOR,PRIVILEGES,PROCEDURE,PUBLIC,"\
#"READ,REAL,REFERENCES,RELATIVE,RESTRICT,REVOKE,RIGHT,ROLLBACK,ROWS"\
#"SCHEMA,SCROLL,SECOND,SECTION,SELECT,SESSION,SESSION_USER,SET,SIZE,"\
#"SMALLINT,SOME,SPACE,SQL,SQLCA,SQLCODE,SQLERROR,SQLSTATE,SQLWARNING,"\
#"SUBSTRING,SUM,SYSTEM_USER,"\
#"TABLE,TEMPORARY,THEN,TIME,TIMESTAMP,TIMEZONE_HOUR,TIMEZONE_MINUTE,"\
#"TO,TRAILING,TRANSACTION,TRANSLATE,TRANSLATION,TRIM,TRUE,"\
#"UNION,UNIQUE,UNKNOWN,UPDATE,UPPER,USAGE,USER,USING,"\
#"VALUE,VALUES,VARCHAR,VARYING,VIEW,WHEN,WHENEVER,WHERE,WITH,WORK,WRITE,"\
#"YEAR,ZONE"
#def SQLAllocEnv(phenv): return SQLAllocHandleStd(SQL_HANDLE_ENV, SQL_NULL_HANDLE, phenv)
#
#SQL_YEAR = SQL_CODE_YEAR
#SQL_MONTH = SQL_CODE_MONTH
#SQL_DAY = SQL_CODE_DAY
#SQL_HOUR = SQL_CODE_HOUR
#SQL_MINUTE = SQL_CODE_MINUTE
#SQL_SECOND = SQL_CODE_SECOND
#SQL_YEAR_TO_MONTH = SQL_CODE_YEAR_TO_MONTH
#SQL_DAY_TO_HOUR = SQL_CODE_DAY_TO_HOUR
#SQL_DAY_TO_MINUTE = SQL_CODE_DAY_TO_MINUTE
#SQL_DAY_TO_SECOND = SQL_CODE_DAY_TO_SECOND
#SQL_HOUR_TO_MINUTE = SQL_CODE_HOUR_TO_MINUTE
#SQL_HOUR_TO_SECOND = SQL_CODE_HOUR_TO_SECOND
#SQL_MINUTE_TO_SECOND = SQL_CODE_MINUTE_TO_SECOND
#SQL_DATABASE_NAME = 16
#SQL_CONCUR_TIMESTAMP = SQL_CONCUR_ROWVER
#SQL_FETCH_RESUME = 7
#SQL_SCROLL_FORWARD_ONLY = 0L
#SQL_SCROLL_KEYSET_DRIVEN = (-1L)
#SQL_SCROLL_DYNAMIC = (-2L)
#SQL_SCROLL_STATIC = (-3L)
#TRACE_VERSION = 1000
#TRACE_ON = 0x00000001L
#TRACE_VS_EVENT_ON = 0x00000002L
#ODBC_VS_FLAG_UNICODE_ARG = 0x00000001L
#ODBC_VS_FLAG_UNICODE_COR = 0x00000002L
#ODBC_VS_FLAG_RETCODE = 0x00000004L
#ODBC_VS_FLAG_STOP = 0x00000008L



# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; version 2 of the
# License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA

from __future__ import with_statement
from workbench.log import log_info, log_error, log_warning, log_debug3

from workbench.utils import format_duration
from workbench.db_utils import QueryError
import mforms
import time
import wb_admin_monitor

def stradd(table, y, label, value):
    t = mforms.newLabel(label)
    table.add(t, 0, 1, y, y+1, mforms.HFillFlag)

    t = mforms.newLabel(value)
    t.set_style(mforms.BoldStyle)
    t.set_color("#555555")
    table.add(t, 1, 2, y, y+1, mforms.HFillFlag)
    return t



class StateIcon(mforms.Box):
    on_icon = None
    off_icon = None

    def __init__(self):
        mforms.Box.__init__(self, True)
        self.set_release_on_add()
        self.set_managed()

        if not self.on_icon:
            self.on_icon = mforms.App.get().get_resource_path("mysql-status-on.png")
            self.off_icon = mforms.App.get().get_resource_path("mysql-status-off.png")

        self.set_spacing(8)
        self.image = mforms.newImageBox()
        self.image.set_image(self.off_icon)
        self.add(self.image, False, True)

        self.label = mforms.newLabel("n/a")
        self.add(self.label, False, True)

        self.text = None


    def set_state(self, state):
        if state:
            self.image.set_image(self.on_icon)
            self.label.set_text("On")
        elif state is None:
            self.image.set_image(self.off_icon)
            self.label.set_text("n/a")
        else:
            self.image.set_image(self.off_icon)
            self.label.set_text("Off")


    def set_text(self, text):
        if not self.text:
            self.text = mforms.newLabel(text)
            self.text.set_style(mforms.BoldStyle)
            self.text.set_color("#555555")
            self.add(self.text, False, True)
        else:
            self.text.set_text(text)



class ConnectionInfo(mforms.Box):
    def __init__(self, owner):
        mforms.Box.__init__(self, True)
        self.set_release_on_add()
        self.set_managed()
        
        self.owner = owner

        self.set_spacing(35)

        self.icon = mforms.newImageBox()
        self.icon.set_image(mforms.App.get().get_resource_path("mysql-logo-00.png"))

        self.add(self.icon, False, True)

        vbox = mforms.newBox(False)
        self.vbox = vbox
        self.add(vbox, True, True)
        vbox.set_spacing(2)
        vbox.add(mforms.newLabel("Connection Name"), False, True)

        self.connection_name = mforms.newLabel("?")
        self.connection_name.set_style(mforms.VeryBigStyle)
        vbox.add(self.connection_name, False, True)

        self.info_table = None


    def update(self, ctrl_be):
        self.suspend_layout()
        self.connection_name.set_text(ctrl_be.server_profile.name)

        info = ctrl_be.server_variables
        status = ctrl_be.status_variables

        if self.info_table:
            self.vbox.remove(self.info_table)

        self.info_table = mforms.newTable()
        self.info_table.set_column_count(2)
        self.info_table.set_row_count(8)
        self.info_table.set_column_spacing(18)
        self.info_table.set_row_spacing(5)
        self.vbox.add(self.info_table, True, True)

        stradd(self.info_table, 0, "\nHost:", "\n"+info.get("hostname", "n/a"))
        stradd(self.info_table, 1, "Socket:", info.get("socket", "n/a"))
        stradd(self.info_table, 2, "Port:", info.get("port", "n/a"))
        stradd(self.info_table, 3, "Version:", "%s\n%s" % (info.get("version", "n/a"), info.get("version_comment", "")))
        stradd(self.info_table, 4, "Compiled For:", "%s   (%s)" % (info.get("version_compile_os", "n/a"), info.get("version_compile_machine", "n/a")))

        stradd(self.info_table, 5, "Configuration File:", ctrl_be.server_profile.config_file_path or "unknown")

        uptime = status.get("Uptime", None)
        if uptime:
            uptime = long(uptime)
            stradd(self.info_table, 6, "Running Since:", "%s (%s)" % (time.ctime(ctrl_be.status_variables_time-uptime), format_duration(uptime, True)))
        else:
            stradd(self.info_table, 6, "Running Since:", "n/a")

        box = mforms.newBox(True)
        refresh = mforms.newButton()
        refresh.set_text("Refresh")
        refresh.set_tooltip("Refresh server status information")
        refresh.add_clicked_callback(self.owner.refresh_status)
        box.add(refresh, False, False)
        self.info_table.add(box, 1, 2, 7, 8, 0)

        version = ctrl_be.target_version
        if version and info:
            icon = mforms.App.get().get_resource_path("mysql-logo-%i%i.png" % (version.majorNumber, version.minorNumber))
            if icon:
                self.icon.set_image(icon)
        self.resume_layout()



#===============================================================================
#
#===============================================================================
class WbAdminServerStatus(mforms.Box):
    status      = None
    connections = None
    _update_timeout = None

    @classmethod
    def wba_register(cls, admin_context):
        admin_context.register_page(cls, "wba_management", "Server Status", False)

    @classmethod
    def identifier(cls):
        return "admin_server_status"


    #---------------------------------------------------------------------------
    def __init__(self, ctrl_be, server_profile, main_view):
        mforms.Box.__init__(self, True)
        self.set_managed()
        self.set_release_on_add()

        self.ui_created = False

        self.set_spacing(24)

        self.ctrl_be = ctrl_be
        self.server_profile = server_profile
        self.main_view = main_view

        lbox = mforms.newBox(False)
        self.add(lbox, True, True)

        self.connection_info = ConnectionInfo(self)
        self.connection_info.set_padding(24)
        lbox.add(self.connection_info, False, True)

        self.scrollbox = mforms.newScrollPanel(mforms.ScrollPanelDrawBackground)
        self.scrollbox.set_padding(24)
        self.content = mforms.newBox(False)
        self.content.set_padding(20)
        self.content.set_spacing(4)
        self.scrollbox.add(self.content)
        lbox.add(self.scrollbox, True, True)

        image = mforms.newImageBox()
        if self.server_profile.host_os == "linux":
            image.set_image(mforms.App.get().get_resource_path("mysql-status-separator-linux.png"))
        else:
          image.set_image(mforms.App.get().get_resource_path("mysql-status-separator.png"))
        image.set_image_align(mforms.MiddleCenter)
        self.add(image, False, True)

        self.status = wb_admin_monitor.WbAdminMonitor(server_profile, self.ctrl_be)
        self.status.set_size(360, -1)
        self.status.set_padding(0, 24, 24, 24)
        self.add(self.status, False, False)

        self.controls = {}

        self.currently_started = None
        self.ctrl_be.add_me_for_event("server_started", self)
        self.ctrl_be.add_me_for_event("server_offline", self)
        self.ctrl_be.add_me_for_event("server_stopped", self)

        self.connection_info.update(self.ctrl_be)

    #---------------------------------------------------------------------------
    def server_started_event(self):
        if self.currently_started != True:
            self.refresh("started")
            self.currently_started = True
            if not self._update_timeout:
                self._update_timeout = mforms.Utilities.add_timeout(0.5, self.update)

    def server_offline_event(self):
        if self.currently_started != True:
            self.refresh("offline")
            self.currently_started = True
            if not self._update_timeout:
                self._update_timeout = mforms.Utilities.add_timeout(0.5, self.update)
    #---------------------------------------------------------------------------
    def server_stopped_event(self):
        if self.currently_started != False:
            self.refresh("stopped")
            self.currently_started = False
            if not self._update_timeout:
                self._update_timeout = mforms.Utilities.add_timeout(0.5, self.update)

    #---------------------------------------------------------------------------
    def refresh(self, status):
        self.status.refresh_status(status)


    #---------------------------------------------------------------------------
    def refresh_status(self):
        if not self._update_timeout:
            status = self.ctrl_be.force_check_server_state()
            if (status == "running" or not status  or status == "offline" ) and self.currently_started:
                self.ctrl_be.query_server_info()

            self._update_timeout = mforms.Utilities.add_timeout(0.5, self.update)


    #---------------------------------------------------------------------------
    def page_activated(self):
        self.suspend_layout()
        try:
            if not self.ui_created:
                self.create_info_sections()
                self.ui_created = True

                if not self._update_timeout:
                    self._update_timeout = mforms.Utilities.add_timeout(0.5, self.update)
        finally:
            self.resume_layout()

        if self.currently_started is None:
            if self.ctrl_be.is_server_running() == "running":
                self.server_started_event()
            elif self.ctrl_be.is_server_running() == "offline":
                self.server_offline_event()
            else:
                self.server_stopped_event()
        else:
            self.ctrl_be.query_server_info()

        self.refresh(self.ctrl_be.is_server_running())

    def update(self):
        self._update_timeout = None
        self.connection_info.update(self.ctrl_be)
        self.status.refresh_status(self.ctrl_be.is_server_running(verbose=False))
        info = self.ctrl_be.server_variables
        status = self.ctrl_be.status_variables
        plugins = dict(self.ctrl_be.server_active_plugins) # plugin -> type

        repl_error = None
        res = None
        try:
            res = self.ctrl_be.exec_query("SHOW SLAVE STATUS")
        except QueryError, e:
            if e.error == 1227:
                repl_error = "Insufficient privileges to view slave status"
            else:
                repl_error = "Error querying status: %s" % str(e)
        repl = {}
        if res and res.nextRow():
            for field in ["Slave_IO_State", "Master_Host"]:
                repl[field] = res.stringByName(field)

        disk_space = "unable to retrieve"
        if self.ctrl_be.server_control and info.get("datadir"):
            disk_space = self.ctrl_be.server_helper.get_available_space(info.get("datadir"))

        # Update the controls in the UI
        self.suspend_layout()
        self.controls["Disk Space in Data Dir:"][0].set_text(disk_space)

        table = self.controls["Replication Slave"][0]

        if repl:
            table.remove(self.controls[""][0])
            self.setup_info_table(table,
                                  [("Slave IO State:", repl.get("Slave_IO_State")),
                                   ("Master Host:", repl.get("Master_Host")),
                                   ("GTID Mode:", info.get("gtid_mode"))],
                                  plugins)
        else:
            self.controls[""][0].set_text(repl_error or "this server is not a slave in a replication setup")
        table.relayout()

        for key, (control, value_source) in self.controls.items():
            if callable(value_source):
                if isinstance(control, mforms.Label):
                    resp = value_source(info, plugins, status)
                    control.set_text(resp if resp else "n/a")
                else:
                    value = value_source(info, plugins, status)
                    if type(value) is tuple:
                        control.set_state(value[0])
                        if value[0] and value[1]:
                            control.set_text(value[1])
                    else:
                        control.set_state(value)

        self.resume_layout()

        mforms.Utilities.driver_shutdown()
        return False

    def create_info_sections(self):
        info = self.ctrl_be.server_variables
        status = self.ctrl_be.status_variables
        plugins = dict(self.ctrl_be.server_active_plugins) # plugin -> type

        repl = {}
        disk_space = "checking..."

        def tristate(value, true_value = None):
            if true_value is not None and value == true_value:
                return True
            if value == "OFF" or value == "NO":
                return False
            elif value and true_value is None:
                return True
            return None

        semi_sync_master = tristate(info.get("rpl_semi_sync_master_enabled"))
        semi_sync_slave = tristate(info.get("rpl_semi_sync_slave_enabled"))
        semi_sync_status = (semi_sync_master or semi_sync_slave, "(%s)"% ", ".join([x for x in [semi_sync_master and "master", semi_sync_slave and "slave"] if x]))
        memcached_status = True if plugins.has_key('daemon_memcached') else None
        
        if not repl:
            if semi_sync_master:
                semi_sync_master = False
            if semi_sync_slave:
                semi_sync_slave = False

        # the params to be passed to the lambdas
        params = (info, plugins, status)

        self.add_info_section_2("Available Server Features",
                              [("Performance Schema:", lambda info, plugins, status: tristate(info.get("performance_schema"))),
                               ("Thread Pool:", lambda info, plugins, status: tristate(info.get("thread_handling"), "loaded-dynamically")),
                               ("Memcached Plugin:", lambda info, plugins, status: memcached_status),
                               ("Semisync Replication Plugin:", lambda info, plugins, status: semi_sync_status),
                               ("SSL Availability:", lambda info, plugins, status: info.get("have_openssl") == "YES" or info.get("have_ssl") == "YES"),
                               ("Windows Authentication:", lambda info, plugins, status: plugins.has_key("authentication_windows")) if self.server_profile.target_is_windows else ("PAM Authentication:", lambda info, plugins, status: plugins.has_key("authentication_pam")),
                               ("Password Validation:", lambda info, plugins, status: (tristate(info.get("validate_password_policy")), "(Policy: %s)" % info.get("validate_password_policy"))),
                               ("Audit Log:", lambda info, plugins, status: (tristate(info.get("audit_log_policy")), "(Log Policy: %s)" % info.get("audit_log_policy"))),
                               ("Firewall:", lambda info, plugins, status: tristate(info.get("mysql_firewall_mode"))),
                               ("Firewall Trace:", lambda info, plugins, status: tristate(info.get("mysql_firewall_trace")))],
                                params)

        log_output = info.get("log_output", "FILE")

        self.add_info_section("Server Directories",
                              [("Base Directory:", lambda info, plugins, status: info.get("basedir")),
                               ("Data Directory:", lambda info, plugins, status: info.get("datadir")),
                               ("Disk Space in Data Dir:", disk_space),
                               ("InnoDB Data Directory:", lambda info, plugins, status: info.get("innodb_data_home_dir")) if info.get("innodb_data_home_dir") else None,
                               ("Plugins Directory:", lambda info, plugins, status: info.get("plugin_dir")),
                               ("Tmp Directory:", lambda info, plugins, status: info.get("tmpdir")),
                               ("Error Log:", lambda info, plugins, status: (info.get("log_error") and info.get("log_error")!="OFF", info.get("log_error"))),
                               ("General Log:", lambda info, plugins, status: (info.get("general_log")!="OFF" and log_output != "NONE", info.get("general_log_file") if "FILE" in log_output else "[Stored in database]")),
                               ("Slow Query Log:", lambda info, plugins, status: (info.get("slow_query_log")!="OFF" and log_output != "NONE", info.get("slow_query_log_file") if "FILE" in log_output else "[Stored in database]"))],
                              params)

        self.add_info_section("Replication Slave",
                              [("", "checking...")],
                              params)

        self.add_info_section("Authentication",
                              [("SHA256 password private key:", lambda info, plugins, status: info.get("sha256_password_private_key_path")),
                               ("SHA256 password public key:", lambda info, plugins, status: info.get("sha256_password_public_key_path"))],
                              params)

        self.add_info_section("SSL",
                              [("SSL CA:", lambda info, plugins, status: info.get("ssl_ca") or "n/a"),
                               ("SSL CA path:", lambda info, plugins, status: info.get("ssl_capath") or "n/a"),
                               ("SSL Cert:", lambda info, plugins, status: info.get("ssl_cert") or "n/a"),
                               ("SSL Cipher:", lambda info, plugins, status: info.get("ssl_cipher") or "n/a"),
                               ("SSL CRL:", lambda info, plugins, status: info.get("ssl_crl") or "n/a"),
                               ("SSL CRL path:", lambda info, plugins, status: info.get("ssl_crlpath") or "n/a"),
                               ("SSL Key:", lambda info, plugins, status: info.get("ssl_key") or "n/a")],
                              params)

        log_debug3("mysql_firewall_trace: %s\n" % info.get("mysql_firewall_trace"))
        log_debug3("Firewall_access_denied: %s\n" % status.get("Firewall_access_denied"))
        log_debug3("Firewall_access_granted: %s\n" % status.get("Firewall_access_granted"))
        log_debug3("Firewall_cached_entries: %s\n" % status.get("Firewall_cached_entries"))

        if info.get("mysql_firewall_mode") == "ON":
            self.add_info_section("Firewall",
                                  [("Access denied:", lambda info, plugins, status: str(status.get("Firewall_access_denied")) or "n/a"),
                                  ("Access granted:", lambda info, plugins, status: str(status.get("Firewall_access_granted")) or "n/a"),
                                  ("Access suspicious:", lambda info, plugins, status: str(status.get("Firewall_access_suspicious")) or "n/a"),
                                  ("Cached entries:", lambda info, plugins, status: str(status.get("Firewall_cached_entries")) or "n/a")],
                                  params)

    def add_info_section_2(self, title, info, params):
        label = mforms.newLabel(title)
        label.set_style(mforms.BigBoldStyle)
        label.set_color("#5f5f5f")
        self.content.add(label, False, True)
        sep = mforms.newBox(False)
        sep.set_back_color("#b2b2b2")
        sep.set_size(-1, 1)
        self.content.add(sep, False, True)

        hbox = mforms.newBox(True)

        info_table = self.make_info_table(info[:len(info)/2], params)
        hbox.add(info_table, True, True)
        info_table = self.make_info_table(info[len(info)/2:], params)
        hbox.add(info_table, True, True)

        self.content.add(hbox, False, True)


    def add_info_section(self, title, info, params):
        label = mforms.newLabel(title)
        label.set_style(mforms.BigBoldStyle)
        label.set_color("#5f5f5f")
        self.content.add(label, False, True)
        sep = mforms.newBox(False)
        sep.set_back_color("#b2b2b2")
        sep.set_size(-1, 1)
        self.content.add(sep, False, True)

        info_table = self.make_info_table([x for x in info if x], params)
        self.content.add(info_table, False, True)
        self.controls[title] = (info_table, None)


    def make_info_table(self, info, params):
        info_table = mforms.newTable()
        info_table.set_column_spacing(8)
        info_table.set_row_spacing(6)
        info_table.set_column_count(2)
        return self.setup_info_table(info_table, info, params)


    def setup_info_table(self, info_table, info, params):
        info_table.set_row_count(len(info)+1)
        for i, item in enumerate(info):
            (label, value_source) = item
            if callable(value_source):
                value = value_source(*params)
            else:
                value = value_source

            if self.controls.has_key(label):
                info_table.remove(self.controls[label][0])
            else:
                info_table.add(mforms.newLabel(label), 0, 1, i, i+1, mforms.HFillFlag)

            if type(value) is bool or value is None:
                b = StateIcon()
                b.set_state(value)
                info_table.add(b, 1, 2, i, i+1, mforms.HFillFlag|mforms.HExpandFlag)
                self.controls[label] = (b, value_source)
            elif type(value) is tuple:
                b = StateIcon()
                b.set_state(value[0])
                if value[0] and value[1]:
                    b.set_text(value[1])
                info_table.add(b, 1, 2, i, i+1, mforms.HFillFlag|mforms.HExpandFlag)
                self.controls[label] = (b, value_source)
            else:
                l2 = mforms.newLabel(value or "")
                l2.set_style(mforms.BoldStyle)
                l2.set_color("#1c1c1c")
                info_table.add(l2, 1, 2, i, i+1, mforms.HFillFlag|mforms.HExpandFlag)
                self.controls[label] = (l2, value_source)
        info_table.add(mforms.newLabel(""), 0, 1, len(info), len(info)+1, mforms.HFillFlag) # blank space
        return info_table


    #---------------------------------------------------------------------------
    def page_deactivated(self):
        pass
    
    #---------------------------------------------------------------------------
    def shutdown(self):
        if self._update_timeout:
            mforms.Utilities.cancel_timeout(self._update_timeout)
            self._update_timeout = None
        self.status.stop()

